// Copyright (c) Redux Robotics and other contributors.
// This is open source and can be modified and shared under the 3-clause BSD license. 

package com.reduxrobotics.canand;

/**
 * CanandDevice device constants. 
 * 
 * This file is autogenerated by canandmessage, <b>do not hand-edit!</b>
 */
public class CanandDeviceDetails {
    private CanandDeviceDetails() {}

    /**
     * Messages.
     */
    public static class Msg {
        private Msg() {}

        /** Device enumerate response */
        public static final int kEnumerate         = 0xb;

        /** Party mode */
        public static final int kPartyMode         = 0x7;

        /** Status frame */
        public static final int kStatus            = 0x6;

        /** Clear device sticky faults */
        public static final int kClearStickyFaults = 0x5;

        /** setting value report from device */
        public static final int kReportSetting     = 0x4;

        /** update setting on device */
        public static final int kSetSetting        = 0x3;

        /** setting control command */
        public static final int kSettingCommand    = 0x2;

        /**
         * Extracts Device-unique serial number from Enumerate.
         * 
         * @param field data bitfield
         * @return serial as a buf:48
         */
        public static long extractEnumerate_Serial(long field) {
            return field & 0xffffffffffffL;
        }

        /**
         * Extracts Device is in bootloader. from Enumerate.
         * 
         * @param field data bitfield
         * @return is_bootloader as a bool
         */
        public static boolean extractEnumerate_IsBootloader(long field) {
            return ((field >> 48) & 1) > 0;
        }

        /**
         * Extracts Party level. 0 disables the strobe, whereas each increased value up to 10 increases strobe period by 50 ms. from PartyMode.
         * 
         * @param field data bitfield
         * @return party_level as a uint:8
         */
        public static int extractPartyMode_PartyLevel(long field) {
            return ((int) field) & 0xff;
        }

        /**
         * Extracts Device-specific status data. See device pages for more information. from Status.
         * 
         * @param field data bitfield
         * @return dev_specific as a buf:64
         */
        public static long extractStatus_DevSpecific(long field) {
            return field;
        }

        /**
         * Extracts Setting index to write to from ReportSetting.
         * 
         * @param field data bitfield
         * @return address as a enum:SETTING
         */
        public static int extractReportSetting_Address(long field) {
            return ((int) field) & 0xff;
        }

        /**
         * Extracts 6-byte setting value from ReportSetting.
         * 
         * @param field data bitfield
         * @return value as a buf:48
         */
        public static long extractReportSetting_Value(long field) {
            return (field >> 8) & 0xffffffffffffL;
        }

        /**
         * Extracts Setting receive status from ReportSetting.
         * 
         * @param field data bitfield
         * @return flags as a bitset:8
         */
        public static int extractReportSetting_Flags(long field) {
            return ((int) (field >> 56)) & 0xff;
        }

        /**
         * Extracts Setting index to write to from SetSetting.
         * 
         * @param field data bitfield
         * @return address as a enum:SETTING
         */
        public static int extractSetSetting_Address(long field) {
            return ((int) field) & 0xff;
        }

        /**
         * Extracts 6-byte setting value from SetSetting.
         * 
         * @param field data bitfield
         * @return value as a buf:48
         */
        public static long extractSetSetting_Value(long field) {
            return (field >> 8) & 0xffffffffffffL;
        }

        /**
         * Extracts Whether the setting should be set ephemeral from SetSetting_Flags.
         * 
         * @param field data bitfield
         * @return ephemeral as a bool
         */
        public static boolean extractSetSetting_Flags_Ephemeral(long field) {
            return ((field >> 56) & 1) > 0;
        }

        /**
         * Extracts Whether the setting should be held until the next synch barrier from SetSetting_Flags.
         * 
         * @param field data bitfield
         * @return synch_hold as a bool
         */
        public static boolean extractSetSetting_Flags_SynchHold(long field) {
            return ((field >> 57) & 1) > 0;
        }

        /**
         * Extracts Synch message count from SetSetting_Flags.
         * 
         * @param field data bitfield
         * @return synch_msg_count as a uint:4
         */
        public static int extractSetSetting_Flags_SynchMsgCount(long field) {
            return ((int) (field >> 60)) & 0xf;
        }

        /**
         * Extracts Setting command index from SettingCommand.
         * 
         * @param field data bitfield
         * @return control_flag as a enum:SETTING_COMMAND
         */
        public static int extractSettingCommand_ControlFlag(long field) {
            return ((int) field) & 0xff;
        }

        /**
         * Extracts setting index to fetch from SettingCommand.
         * 
         * @param field data bitfield
         * @return setting_index as a enum:SETTING
         */
        public static int extractSettingCommand_SettingIndex(long field) {
            return ((int) (field >> 8)) & 0xff;
        }

        /**
         * Constructs a ENUMERATE message.
         * 
         * @param serial Device-unique serial number (buf:48)
         * @param isBootloader Device is in bootloader. (bool)
         * @return message data as long
         */
        public static long constructEnumerate(long serial, boolean isBootloader) {
            return (serial & 0xffffffffffffL) | 
                ((isBootloader ? 1L : 0L) << 48);
        }

        /**
         * Constructs a PARTY_MODE message.
         * 
         * @param partyLevel Party level. 0 disables the strobe, whereas each increased value up to 10 increases strobe period by 50 ms. (uint:8)
         * @return message data as long
         */
        public static long constructPartyMode(int partyLevel) {
            return ((long) partyLevel);
        }

        /**
         * Constructs a STATUS message.
         * 
         * @param devSpecific Device-specific status data. See device pages for more information. (buf:64)
         * @return message data as long
         */
        public static long constructStatus(long devSpecific) {
            return devSpecific;
        }

        /**
         * Constructs a CLEAR_STICKY_FAULTS message.
         * 
         * 
         * @return message data as long
         */
        public static long constructClearStickyFaults() {
            return 0;
        }

        /**
         * Constructs a REPORT_SETTING message.
         * 
         * @param address Setting index to write to (enum:SETTING)
         * @param value 6-byte setting value (buf:48)
         * @param flags Setting receive status (bitset:8)
         * @return message data as long
         */
        public static long constructReportSetting(int address, long value, int flags) {
            return ((long) address) | 
                ((value & 0xffffffffffffL) << 8) | 
                (((long) flags) << 56);
        }

        /**
         * Constructs a SET_SETTING message.
         * 
         * @param address Setting index to write to (enum:SETTING)
         * @param value 6-byte setting value (buf:48)
         * @param flagsEphemeral Whether the setting should be set ephemeral (bool)
         * @param flagsSynchHold Whether the setting should be held until the next synch barrier (bool)
         * @param flagsSynchMsgCount Synch message count (uint:4)
         * @return message data as long
         */
        public static long constructSetSetting(int address, long value, boolean flagsEphemeral, boolean flagsSynchHold, int flagsSynchMsgCount) {
            return ((long) address) | 
                ((value & 0xffffffffffffL) << 8) | 
                ((flagsEphemeral ? 1L : 0L) << 56) | 
                ((flagsSynchHold ? 1L : 0L) << 57) | 
                (((long) flagsSynchMsgCount) << 60);
        }

        /**
         * Constructs a SETTING_COMMAND message.
         * 
         * @param controlFlag Setting command index (enum:SETTING_COMMAND)
         * @param settingIndex setting index to fetch (enum:SETTING)
         * @return message data as long
         */
        public static long constructSettingCommand(int controlFlag, int settingIndex) {
            return ((long) controlFlag) | 
                (((long) settingIndex) << 8);
        }

        /** ENUMERATE message length */
        public static final int kDlc_Enumerate = 8;

        /** PARTY_MODE message min length */
        public static final int kDlcMin_PartyMode = 1;

        /** PARTY_MODE message max length */
        public static final int kDlcMax_PartyMode = 8;

        /** STATUS message length */
        public static final int kDlc_Status = 8;

        /** CLEAR_STICKY_FAULTS message min length */
        public static final int kDlcMin_ClearStickyFaults = 0;

        /** CLEAR_STICKY_FAULTS message max length */
        public static final int kDlcMax_ClearStickyFaults = 8;

        /** REPORT_SETTING message length */
        public static final int kDlc_ReportSetting = 8;

        /** SET_SETTING message length */
        public static final int kDlc_SetSetting = 8;

        /** SETTING_COMMAND message min length */
        public static final int kDlcMin_SettingCommand = 1;

        /** SETTING_COMMAND message max length */
        public static final int kDlcMax_SettingCommand = 8;
    }

    /**
     * Settings.
     */
    public static class Stg {
        private Stg() {}

        /** Device-specific type identifier */
        public static final int kDeviceType        = 0x8;

        /** Firmware version */
        public static final int kFirmwareVersion   = 0x6;

        /** Serial number */
        public static final int kSerialNumber      = 0x5;

        /** Status frame period (ms) */
        public static final int kStatusFramePeriod = 0x4;

        /**
         * Extracts Device-specific type identifier from DeviceType.
         * 
         * @param field data bitfield
         * @return DEVICE_TYPE as a uint:16
         */
        public static int extractDeviceType(long field) {
            return ((int) field) & 0xffff;
        }

        /**
         * Extracts Firmware version patch number from FirmwareVersion.
         * 
         * @param field data bitfield
         * @return firmware_patch as a uint:8
         */
        public static int extractFirmwareVersion_FirmwarePatch(long field) {
            return ((int) field) & 0xff;
        }

        /**
         * Extracts Firmware version minor number from FirmwareVersion.
         * 
         * @param field data bitfield
         * @return firmware_minor as a uint:8
         */
        public static int extractFirmwareVersion_FirmwareMinor(long field) {
            return ((int) (field >> 8)) & 0xff;
        }

        /**
         * Extracts Firmware version year from FirmwareVersion.
         * 
         * @param field data bitfield
         * @return firmware_year as a uint:16
         */
        public static int extractFirmwareVersion_FirmwareYear(long field) {
            return ((int) (field >> 16)) & 0xffff;
        }

        /**
         * Extracts Serial number from SerialNumber.
         * 
         * @param field data bitfield
         * @return SERIAL_NUMBER as a buf:48
         */
        public static long extractSerialNumber(long field) {
            return field & 0xffffffffffffL;
        }

        /**
         * Extracts Status frame period (ms) from StatusFramePeriod.
         * 
         * @param field data bitfield
         * @return STATUS_FRAME_PERIOD as a uint:16
         */
        public static int extractStatusFramePeriod(long field) {
            return ((int) field) & 0xffff;
        }

        /**
         * Constructs a DEVICE_TYPE setting.
         * 
         * @param deviceType Device-specific type identifier (uint:16)
         * @return message data as long
         */
        public static long constructDeviceType(int deviceType) {
            if (deviceType < 0 || deviceType > 65535) { throw new IllegalArgumentException("deviceType must be between [0..=65535] inclusive, instead got " + deviceType); }
            return ((long) deviceType);
        }

        /**
         * Constructs a FIRMWARE_VERSION setting.
         * 
         * @param firmwarePatch Firmware version patch number (uint:8)
         * @param firmwareMinor Firmware version minor number (uint:8)
         * @param firmwareYear Firmware version year (uint:16)
         * @return message data as long
         */
        public static long constructFirmwareVersion(int firmwarePatch, int firmwareMinor, int firmwareYear) {
            if (firmwarePatch < 0 || firmwarePatch > 255) { throw new IllegalArgumentException("firmwarePatch must be between [0..=255] inclusive, instead got " + firmwarePatch); }
            if (firmwareMinor < 0 || firmwareMinor > 255) { throw new IllegalArgumentException("firmwareMinor must be between [0..=255] inclusive, instead got " + firmwareMinor); }
            if (firmwareYear < 0 || firmwareYear > 65535) { throw new IllegalArgumentException("firmwareYear must be between [0..=65535] inclusive, instead got " + firmwareYear); }
            return ((long) firmwarePatch) | 
                (((long) firmwareMinor) << 8) | 
                (((long) firmwareYear) << 16);
        }

        /**
         * Constructs a SERIAL_NUMBER setting.
         * 
         * @param serialNumber Serial number (buf:48)
         * @return message data as long
         */
        public static long constructSerialNumber(long serialNumber) {
            if (serialNumber < 0L || serialNumber > 281474976710655L) { throw new IllegalArgumentException("serialNumber must be between [0..=281474976710655] inclusive, instead got " + serialNumber); }
            return (serialNumber & 0xffffffffffffL);
        }

        /**
         * Constructs a STATUS_FRAME_PERIOD setting.
         * 
         * @param statusFramePeriod Status frame period (ms) (uint:16)
         * @return message data as long
         */
        public static long constructStatusFramePeriod(int statusFramePeriod) {
            if (statusFramePeriod < 1 || statusFramePeriod > 16383) { throw new IllegalArgumentException("statusFramePeriod must be between [1..=16383] inclusive, instead got " + statusFramePeriod); }
            return ((long) statusFramePeriod);
        }

        /** List of settings to fetch for. */
        public static int settingsAddresses[] = {
            kStatusFramePeriod,
            kSerialNumber,
            kFirmwareVersion,
            kDeviceType,
        };
    }

    /**
     * Cananddevice enums.
     */
    public static class Enums {
        private Enums() {}

        /**
         * enum CanandDevice::SETTING_COMMAND.
         */
        public static class SettingCommand {
            private SettingCommand() {}

            /** Fetch all settings from device */
            public static final int kFetchSettings       = 0x0;

            /** Reset settings to factory default */
            public static final int kResetFactoryDefault = 0x1;

            /** Fetches a single setting from device */
            public static final int kFetchSettingValue   = 0x2;
        }
    }

    /**
     * Cananddevice bitsets.
     */
    public static class Bitsets {
        private Bitsets() {}

        /**
         * setting_report_flags - Whether the setting set was successful
         */
        public static final int kSettingReportFlags_SetSuccess = 0x1;

        /**
         * setting_report_flags - Whether the setting synch commit was successful
         */
        public static final int kSettingReportFlags_CommitSuccess = 0x2;

        /**
         * Constructs a setting_report_flags bitset.
         * 
         * @param setSuccess Whether the setting set was successful
         * @param commitSuccess Whether the setting synch commit was successful
         * @return bitset data as int
         */
        public static int constructSettingReportFlags(boolean setSuccess, boolean commitSuccess) {
                return (setSuccess ? 0x1 : 0) | 
                (commitSuccess ? 0x2 : 0);
        }

        /**
         * Extracts set_success from setting_report_flags.
         * 
         * @param field data bitfield
         * @return true if set, false if not
         */
        public static boolean extractSettingReportFlags_SetSuccess(int field) {
            return (field & kSettingReportFlags_SetSuccess) > 0;
        }

        /**
         * Extracts commit_success from setting_report_flags.
         * 
         * @param field data bitfield
         * @return true if set, false if not
         */
        public static boolean extractSettingReportFlags_CommitSuccess(int field) {
            return (field & kSettingReportFlags_CommitSuccess) > 0;
        }
    }
}