// Copyright (c) Redux Robotics and other contributors.
// This is open source and can be modified and shared under the 3-clause BSD license. 

package com.reduxrobotics.sensors.canandcolor;

import java.util.HashMap;
import java.util.Map;

/**
 * Canandcolor device constants. 
 * 
 * This file is autogenerated by canandmessage, &lt;b&gt;do not hand-edit!&lt;/b&gt;
 */
public class CanandcolorDetails {
    private CanandcolorDetails() {}

    /**
     * Messages.
     */
    public static class Msg {
        private Msg() {}

        /**
         * Distance frame
         */
        public static final int kDistanceOutput    = 0x1f;

        /**
         * Color frame
         */
        public static final int kColorOutput       = 0x1e;

        /**
         * Digital output frame
         */
        public static final int kDigitalOutput     = 0x1d;

        /**
         * Clear sticky digout state which is broadcast over CAN
         */
        public static final int kClearStickyDigout = 0x1c;

        /**
         * Device enumerate response
         */
        public static final int kEnumerate         = 0xb;

        /**
         * Party mode
         */
        public static final int kPartyMode         = 0x7;

        /**
         * Status frame
         */
        public static final int kStatus            = 0x6;

        /**
         * Clear device sticky faults
         */
        public static final int kClearStickyFaults = 0x5;

        /**
         * setting value report from device
         */
        public static final int kReportSetting     = 0x4;

        /**
         * update setting on device
         */
        public static final int kSetSetting        = 0x3;

        /**
         * setting control command
         */
        public static final int kSettingCommand    = 0x2;

        /**
         * Extracts 16-bit distance value. Actual correspondance to real-world units is config and surface-dependent. from DistanceOutput.
         * 
         * @param field data bitfield
         * @return distance as a uint:16
         */
        public static int extractDistanceOutput_Distance(long field) {
            return ((int) field) & 0xffff;
        }

        /**
         * Extracts Red reading magnitude from ColorOutput.
         * 
         * @param field data bitfield
         * @return red as a uint:20
         */
        public static int extractColorOutput_Red(long field) {
            return ((int) field) & 0xfffff;
        }

        /**
         * Extracts Green reading magnitude from ColorOutput.
         * 
         * @param field data bitfield
         * @return green as a uint:20
         */
        public static int extractColorOutput_Green(long field) {
            return ((int) (field >> 20)) & 0xfffff;
        }

        /**
         * Extracts Blue reading magnitude from ColorOutput.
         * 
         * @param field data bitfield
         * @return blue as a uint:20
         */
        public static int extractColorOutput_Blue(long field) {
            return ((int) (field >> 40)) & 0xfffff;
        }

        /**
         * Extracts Color integration period from ColorOutput.
         * 
         * @param field data bitfield
         * @return period as a enum:COLOR_INTEGRATION_PERIOD
         */
        public static int extractColorOutput_Period(long field) {
            return ((int) (field >> 60)) & 0xf;
        }

        /**
         * Extracts Digital output state for DIGOUT1 from DigitalOutput.
         * 
         * @param field data bitfield
         * @return digout1_state as a bool
         */
        public static boolean extractDigitalOutput_Digout1State(long field) {
            return ((field >> 0) & 1) > 0;
        }

        /**
         * Extracts Digital output state for DIGOUT2 from DigitalOutput.
         * 
         * @param field data bitfield
         * @return digout2_state as a bool
         */
        public static boolean extractDigitalOutput_Digout2State(long field) {
            return ((field >> 1) & 1) > 0;
        }

        /**
         * Extracts Sticky digital output state for DIGOUT1 from DigitalOutput.
         * 
         * @param field data bitfield
         * @return digout1_sticky as a bool
         */
        public static boolean extractDigitalOutput_Digout1Sticky(long field) {
            return ((field >> 2) & 1) > 0;
        }

        /**
         * Extracts Sticky digital output state for DIGOUT1 from DigitalOutput.
         * 
         * @param field data bitfield
         * @return digout2_sticky as a bool
         */
        public static boolean extractDigitalOutput_Digout2Sticky(long field) {
            return ((field >> 3) & 1) > 0;
        }

        /**
         * Extracts DIGOUT1 condition slot flags. A value of 1 for bit N means that condition slot is true. Bits are indexed little-endian. from DigitalOutput.
         * 
         * @param field data bitfield
         * @return digout1_cond as a bitset:16
         */
        public static int extractDigitalOutput_Digout1Cond(long field) {
            return ((int) (field >> 8)) & 0xffff;
        }

        /**
         * Extracts DIGOUT2 condition slot flags. A value of 1 for bit N means that condition slot is true. Bits are indexed little-endian. from DigitalOutput.
         * 
         * @param field data bitfield
         * @return digout2_cond as a bitset:16
         */
        public static int extractDigitalOutput_Digout2Cond(long field) {
            return ((int) (field >> 24)) & 0xffff;
        }

        /**
         * Extracts Device-unique serial number from Enumerate.
         * 
         * @param field data bitfield
         * @return serial as a buf:48
         */
        public static long extractEnumerate_Serial(long field) {
            return field & 0xffffffffffffL;
        }

        /**
         * Extracts Device is in bootloader. from Enumerate.
         * 
         * @param field data bitfield
         * @return is_bootloader as a bool
         */
        public static boolean extractEnumerate_IsBootloader(long field) {
            return ((field >> 48) & 1) > 0;
        }

        /**
         * Extracts Party level. 0 disables the strobe, whereas each increased value up to 10 increases strobe period by 50 ms. from PartyMode.
         * 
         * @param field data bitfield
         * @return party_level as a uint:8
         */
        public static int extractPartyMode_PartyLevel(long field) {
            return ((int) field) & 0xff;
        }

        /**
         * Extracts 8-bit active faults bitfield from Status.
         * 
         * @param field data bitfield
         * @return faults as a bitset:8
         */
        public static int extractStatus_Faults(long field) {
            return ((int) field) & 0xff;
        }

        /**
         * Extracts 8-bit sticky faults bitfield from Status.
         * 
         * @param field data bitfield
         * @return sticky_faults as a bitset:8
         */
        public static int extractStatus_StickyFaults(long field) {
            return ((int) (field >> 8)) & 0xff;
        }

        /**
         * Extracts 16-bit signed temperature byte in 1/256ths of a Celsius from Status.
         * 
         * @param field data bitfield
         * @return temperature as a sint:16
         */
        public static int extractStatus_Temperature(long field) {
            return ((((int) (field >> 16)) & 0xffff) << 16) >> 16;
        }

        /**
         * Extracts Setting index to write to from ReportSetting.
         * 
         * @param field data bitfield
         * @return address as a enum:SETTING
         */
        public static int extractReportSetting_Address(long field) {
            return ((int) field) & 0xff;
        }

        /**
         * Extracts 6-byte setting value from ReportSetting.
         * 
         * @param field data bitfield
         * @return value as a buf:48
         */
        public static long extractReportSetting_Value(long field) {
            return (field >> 8) & 0xffffffffffffL;
        }

        /**
         * Extracts Setting receive status from ReportSetting.
         * 
         * @param field data bitfield
         * @return flags as a bitset:8
         */
        public static int extractReportSetting_Flags(long field) {
            return ((int) (field >> 56)) & 0xff;
        }

        /**
         * Extracts Setting index to write to from SetSetting.
         * 
         * @param field data bitfield
         * @return address as a enum:SETTING
         */
        public static int extractSetSetting_Address(long field) {
            return ((int) field) & 0xff;
        }

        /**
         * Extracts 6-byte setting value from SetSetting.
         * 
         * @param field data bitfield
         * @return value as a buf:48
         */
        public static long extractSetSetting_Value(long field) {
            return (field >> 8) & 0xffffffffffffL;
        }

        /**
         * Extracts Whether the setting should be set ephemeral from SetSetting_Flags.
         * 
         * @param field data bitfield
         * @return ephemeral as a bool
         */
        public static boolean extractSetSetting_Flags_Ephemeral(long field) {
            return ((field >> 56) & 1) > 0;
        }

        /**
         * Extracts Whether the setting should be held until the next synch barrier from SetSetting_Flags.
         * 
         * @param field data bitfield
         * @return synch_hold as a bool
         */
        public static boolean extractSetSetting_Flags_SynchHold(long field) {
            return ((field >> 57) & 1) > 0;
        }

        /**
         * Extracts Synch message count from SetSetting_Flags.
         * 
         * @param field data bitfield
         * @return synch_msg_count as a uint:4
         */
        public static int extractSetSetting_Flags_SynchMsgCount(long field) {
            return ((int) (field >> 60)) & 0xf;
        }

        /**
         * Extracts Setting command index from SettingCommand.
         * 
         * @param field data bitfield
         * @return control_flag as a enum:SETTING_COMMAND
         */
        public static int extractSettingCommand_ControlFlag(long field) {
            return ((int) field) & 0xff;
        }

        /**
         * Extracts setting index to fetch from SettingCommand.
         * 
         * @param field data bitfield
         * @return setting_index as a enum:SETTING
         */
        public static int extractSettingCommand_SettingIndex(long field) {
            return ((int) (field >> 8)) & 0xff;
        }

        /**
         * Constructs a DISTANCE_OUTPUT message.
         * 
         * @param distance 16-bit distance value. Actual correspondance to real-world units is config and surface-dependent. (uint:16)
         * @return message data as long
         */
        public static long constructDistanceOutput(int distance) {
            return ((long) distance);
        }

        /**
         * Constructs a COLOR_OUTPUT message.
         * 
         * @param red Red reading magnitude (uint:20)
         * @param green Green reading magnitude (uint:20)
         * @param blue Blue reading magnitude (uint:20)
         * @param period Color integration period (enum:COLOR_INTEGRATION_PERIOD)
         * @return message data as long
         */
        public static long constructColorOutput(int red, int green, int blue, int period) {
            return ((long) red) | 
                (((long) green) << 20) | 
                (((long) blue) << 40) | 
                (((long) period) << 60);
        }

        /**
         * Constructs a DIGITAL_OUTPUT message.
         * 
         * @param digout1State Digital output state for DIGOUT1 (bool)
         * @param digout2State Digital output state for DIGOUT2 (bool)
         * @param digout1Sticky Sticky digital output state for DIGOUT1 (bool)
         * @param digout2Sticky Sticky digital output state for DIGOUT1 (bool)
         * @param digout1Cond DIGOUT1 condition slot flags. A value of 1 for bit N means that condition slot is true. Bits are indexed little-endian. (bitset:16)
         * @param digout2Cond DIGOUT2 condition slot flags. A value of 1 for bit N means that condition slot is true. Bits are indexed little-endian. (bitset:16)
         * @return message data as long
         */
        public static long constructDigitalOutput(boolean digout1State, boolean digout2State, boolean digout1Sticky, boolean digout2Sticky, int digout1Cond, int digout2Cond) {
            return (digout1State ? 1L : 0L) | 
                ((digout2State ? 1L : 0L) << 1) | 
                ((digout1Sticky ? 1L : 0L) << 2) | 
                ((digout2Sticky ? 1L : 0L) << 3) | 
                (((long) digout1Cond) << 8) | 
                (((long) digout2Cond) << 24);
        }

        /**
         * Constructs a CLEAR_STICKY_DIGOUT message.
         * 
         * 
         * @return message data as long
         */
        public static long constructClearStickyDigout() {
            return 0;
        }

        /**
         * Constructs a ENUMERATE message.
         * 
         * @param serial Device-unique serial number (buf:48)
         * @param isBootloader Device is in bootloader. (bool)
         * @return message data as long
         */
        public static long constructEnumerate(long serial, boolean isBootloader) {
            return (serial & 0xffffffffffffL) | 
                ((isBootloader ? 1L : 0L) << 48);
        }

        /**
         * Constructs a PARTY_MODE message.
         * 
         * @param partyLevel Party level. 0 disables the strobe, whereas each increased value up to 10 increases strobe period by 50 ms. (uint:8)
         * @return message data as long
         */
        public static long constructPartyMode(int partyLevel) {
            return ((long) partyLevel);
        }

        /**
         * Constructs a STATUS message.
         * 
         * @param faults 8-bit active faults bitfield (bitset:8)
         * @param stickyFaults 8-bit sticky faults bitfield (bitset:8)
         * @param temperature 16-bit signed temperature byte in 1/256ths of a Celsius (sint:16)
         * @return message data as long
         */
        public static long constructStatus(int faults, int stickyFaults, int temperature) {
            return ((long) faults) | 
                (((long) stickyFaults) << 8) | 
                (((long) temperature) << 16);
        }

        /**
         * Constructs a CLEAR_STICKY_FAULTS message.
         * 
         * 
         * @return message data as long
         */
        public static long constructClearStickyFaults() {
            return 0;
        }

        /**
         * Constructs a REPORT_SETTING message.
         * 
         * @param address Setting index to write to (enum:SETTING)
         * @param value 6-byte setting value (buf:48)
         * @param flags Setting receive status (bitset:8)
         * @return message data as long
         */
        public static long constructReportSetting(int address, long value, int flags) {
            return ((long) address) | 
                ((value & 0xffffffffffffL) << 8) | 
                (((long) flags) << 56);
        }

        /**
         * Constructs a SET_SETTING message.
         * 
         * @param address Setting index to write to (enum:SETTING)
         * @param value 6-byte setting value (buf:48)
         * @param flagsEphemeral Whether the setting should be set ephemeral (bool)
         * @param flagsSynchHold Whether the setting should be held until the next synch barrier (bool)
         * @param flagsSynchMsgCount Synch message count (uint:4)
         * @return message data as long
         */
        public static long constructSetSetting(int address, long value, boolean flagsEphemeral, boolean flagsSynchHold, int flagsSynchMsgCount) {
            return ((long) address) | 
                ((value & 0xffffffffffffL) << 8) | 
                ((flagsEphemeral ? 1L : 0L) << 56) | 
                ((flagsSynchHold ? 1L : 0L) << 57) | 
                (((long) flagsSynchMsgCount) << 60);
        }

        /**
         * Constructs a SETTING_COMMAND message.
         * 
         * @param controlFlag Setting command index (enum:SETTING_COMMAND)
         * @param settingIndex setting index to fetch (enum:SETTING)
         * @return message data as long
         */
        public static long constructSettingCommand(int controlFlag, int settingIndex) {
            return ((long) controlFlag) | 
                (((long) settingIndex) << 8);
        }

        /** DISTANCE_OUTPUT message length */
        public static final int kDlc_DistanceOutput = 2;

        /**
         * Check if DISTANCE_OUTPUT message length is valid.
         * @param dlc length to check
         * @return true if valid
         */
        public static boolean checkDlcForDistanceOutput(int dlc) {
            return dlc == 2;
        }

        /** COLOR_OUTPUT message length */
        public static final int kDlc_ColorOutput = 8;

        /**
         * Check if COLOR_OUTPUT message length is valid.
         * @param dlc length to check
         * @return true if valid
         */
        public static boolean checkDlcForColorOutput(int dlc) {
            return dlc == 8;
        }

        /** DIGITAL_OUTPUT message length */
        public static final int kDlc_DigitalOutput = 5;

        /**
         * Check if DIGITAL_OUTPUT message length is valid.
         * @param dlc length to check
         * @return true if valid
         */
        public static boolean checkDlcForDigitalOutput(int dlc) {
            return dlc == 5;
        }

        /** CLEAR_STICKY_DIGOUT message length */
        public static final int kDlc_ClearStickyDigout = 0;

        /**
         * Check if CLEAR_STICKY_DIGOUT message length is valid.
         * @param dlc length to check
         * @return true if valid
         */
        public static boolean checkDlcForClearStickyDigout(int dlc) {
            return dlc == 0;
        }

        /** ENUMERATE message length */
        public static final int kDlc_Enumerate = 8;

        /**
         * Check if ENUMERATE message length is valid.
         * @param dlc length to check
         * @return true if valid
         */
        public static boolean checkDlcForEnumerate(int dlc) {
            return dlc == 8;
        }

        /** PARTY_MODE message min length */
        public static final int kDlcMin_PartyMode = 1;

        /** PARTY_MODE message max length */
        public static final int kDlcMax_PartyMode = 8;

        /**
         * Check if PARTY_MODE message length is valid.
         * @param dlc length to check
         * @return true if valid
         */
        public static boolean checkDlcForPartyMode(int dlc) {
            return dlc >= 1 && dlc <= 8;
        }

        /** STATUS message length */
        public static final int kDlc_Status = 8;

        /**
         * Check if STATUS message length is valid.
         * @param dlc length to check
         * @return true if valid
         */
        public static boolean checkDlcForStatus(int dlc) {
            return dlc == 8;
        }

        /** CLEAR_STICKY_FAULTS message min length */
        public static final int kDlcMin_ClearStickyFaults = 0;

        /** CLEAR_STICKY_FAULTS message max length */
        public static final int kDlcMax_ClearStickyFaults = 8;

        /**
         * Check if CLEAR_STICKY_FAULTS message length is valid.
         * @param dlc length to check
         * @return true if valid
         */
        public static boolean checkDlcForClearStickyFaults(int dlc) {
            return dlc >= 0 && dlc <= 8;
        }

        /** REPORT_SETTING message length */
        public static final int kDlc_ReportSetting = 8;

        /**
         * Check if REPORT_SETTING message length is valid.
         * @param dlc length to check
         * @return true if valid
         */
        public static boolean checkDlcForReportSetting(int dlc) {
            return dlc == 8;
        }

        /** SET_SETTING message length */
        public static final int kDlc_SetSetting = 8;

        /**
         * Check if SET_SETTING message length is valid.
         * @param dlc length to check
         * @return true if valid
         */
        public static boolean checkDlcForSetSetting(int dlc) {
            return dlc == 8;
        }

        /** SETTING_COMMAND message min length */
        public static final int kDlcMin_SettingCommand = 1;

        /** SETTING_COMMAND message max length */
        public static final int kDlcMax_SettingCommand = 8;

        /**
         * Check if SETTING_COMMAND message length is valid.
         * @param dlc length to check
         * @return true if valid
         */
        public static boolean checkDlcForSettingCommand(int dlc) {
            return dlc >= 1 && dlc <= 8;
        }
    }

    /**
     * Settings.
     */
    public static class Stg {
        private Stg() {}

        /**
         * Distance frame period (ms)
         */
        public static final int kDistanceFramePeriod       = 0xff;

        /**
         * Color frame period (ms)
         */
        public static final int kColorFramePeriod          = 0xfe;

        /**
         * Digout frame period (ms)
         */
        public static final int kDigoutFramePeriod         = 0xfd;

        /**
         * Distance extra frame mode
         */
        public static final int kDistanceExtraFrameMode    = 0xf7;

        /**
         * Color extra frame frame mode
         */
        public static final int kColorExtraFrameMode       = 0xf6;

        /**
         * Lamp LED brightness
         */
        public static final int kLampBrightness            = 0xef;

        /**
         * Color integration period
         */
        public static final int kColorIntegrationPeriod    = 0xee;

        /**
         * Distance integration period
         */
        public static final int kDistanceIntegrationPeriod = 0xed;

        /**
         * Digital output 1 control config
         */
        public static final int kDigout1OutputConfig       = 0xeb;

        /**
         * Digital output 2 control config
         */
        public static final int kDigout2OutputConfig       = 0xea;

        /**
         * Digital output 1 send message on change
         */
        public static final int kDigout1MessageOnChange    = 0xe9;

        /**
         * Digital output 2 send message on change
         */
        public static final int kDigout2MessageOnChange    = 0xe8;

        /**
         * Digout1 config slot 0
         */
        public static final int kDigout1Config0            = 0xd0;

        /**
         * Digout1 config slot 1
         */
        public static final int kDigout1Config1            = 0xcf;

        /**
         * Digout1 config slot 2
         */
        public static final int kDigout1Config2            = 0xce;

        /**
         * Digout1 config slot 3
         */
        public static final int kDigout1Config3            = 0xcd;

        /**
         * Digout1 config slot 4
         */
        public static final int kDigout1Config4            = 0xcc;

        /**
         * Digout1 config slot 5
         */
        public static final int kDigout1Config5            = 0xcb;

        /**
         * Digout1 config slot 6
         */
        public static final int kDigout1Config6            = 0xca;

        /**
         * Digout1 config slot 7
         */
        public static final int kDigout1Config7            = 0xc9;

        /**
         * Digout1 config slot 8
         */
        public static final int kDigout1Config8            = 0xc8;

        /**
         * Digout1 config slot 9
         */
        public static final int kDigout1Config9            = 0xc7;

        /**
         * Digout1 config slot 10
         */
        public static final int kDigout1Config10           = 0xc6;

        /**
         * Digout1 config slot 11
         */
        public static final int kDigout1Config11           = 0xc5;

        /**
         * Digout1 config slot 12
         */
        public static final int kDigout1Config12           = 0xc4;

        /**
         * Digout1 config slot 13
         */
        public static final int kDigout1Config13           = 0xc3;

        /**
         * Digout1 config slot 14
         */
        public static final int kDigout1Config14           = 0xc2;

        /**
         * Digout1 config slot 15
         */
        public static final int kDigout1Config15           = 0xc1;

        /**
         * Digout2 config slot 0
         */
        public static final int kDigout2Config0            = 0xc0;

        /**
         * Digout2 config slot 1
         */
        public static final int kDigout2Config1            = 0xbf;

        /**
         * Digout2 config slot 2
         */
        public static final int kDigout2Config2            = 0xbe;

        /**
         * Digout2 config slot 3
         */
        public static final int kDigout2Config3            = 0xbd;

        /**
         * Digout2 config slot 4
         */
        public static final int kDigout2Config4            = 0xbc;

        /**
         * Digout2 config slot 5
         */
        public static final int kDigout2Config5            = 0xbb;

        /**
         * Digout2 config slot 6
         */
        public static final int kDigout2Config6            = 0xba;

        /**
         * Digout2 config slot 7
         */
        public static final int kDigout2Config7            = 0xb9;

        /**
         * Digout2 config slot 8
         */
        public static final int kDigout2Config8            = 0xb8;

        /**
         * Digout2 config slot 9
         */
        public static final int kDigout2Config9            = 0xb7;

        /**
         * Digout2 config slot 10
         */
        public static final int kDigout2Config10           = 0xb6;

        /**
         * Digout2 config slot 11
         */
        public static final int kDigout2Config11           = 0xb5;

        /**
         * Digout2 config slot 12
         */
        public static final int kDigout2Config12           = 0xb4;

        /**
         * Digout2 config slot 13
         */
        public static final int kDigout2Config13           = 0xb3;

        /**
         * Digout2 config slot 14
         */
        public static final int kDigout2Config14           = 0xb2;

        /**
         * Digout2 config slot 15
         */
        public static final int kDigout2Config15           = 0xb1;

        /**
         * Device-specific type identifier
         */
        public static final int kDeviceType                = 0x8;

        /**
         * Firmware version
         */
        public static final int kFirmwareVersion           = 0x6;

        /**
         * Serial number
         */
        public static final int kSerialNumber              = 0x5;

        /**
         * Status frame period (ms)
         */
        public static final int kStatusFramePeriod         = 0x4;

        /**
         * Extracts Distance frame period (ms) from DistanceFramePeriod.
         * 
         * @param field data bitfield
         * @return DISTANCE_FRAME_PERIOD as a uint:16
         */
        public static int extractDistanceFramePeriod(long field) {
            return ((int) field) & 0xffff;
        }

        /**
         * Extracts Color frame period (ms) from ColorFramePeriod.
         * 
         * @param field data bitfield
         * @return COLOR_FRAME_PERIOD as a uint:16
         */
        public static int extractColorFramePeriod(long field) {
            return ((int) field) & 0xffff;
        }

        /**
         * Extracts Digout frame period (ms) from DigoutFramePeriod.
         * 
         * @param field data bitfield
         * @return DIGOUT_FRAME_PERIOD as a uint:16
         */
        public static int extractDigoutFramePeriod(long field) {
            return ((int) field) & 0xffff;
        }

        /**
         * Extracts Distance extra frame mode from DistanceExtraFrameMode.
         * 
         * @param field data bitfield
         * @return DISTANCE_EXTRA_FRAME_MODE as a enum:EXTRA_FRAME_MODE
         */
        public static int extractDistanceExtraFrameMode(long field) {
            return ((int) field) & 0xff;
        }

        /**
         * Extracts Color extra frame frame mode from ColorExtraFrameMode.
         * 
         * @param field data bitfield
         * @return COLOR_EXTRA_FRAME_MODE as a enum:EXTRA_FRAME_MODE
         */
        public static int extractColorExtraFrameMode(long field) {
            return ((int) field) & 0xff;
        }

        /**
         * Extracts Lamp LED brightness from LampBrightness.
         * 
         * @param field data bitfield
         * @return LAMP_BRIGHTNESS as a uint:16
         */
        public static int extractLampBrightness(long field) {
            return ((int) field) & 0xffff;
        }

        /**
         * Extracts Color integration period from ColorIntegrationPeriod.
         * 
         * @param field data bitfield
         * @return COLOR_INTEGRATION_PERIOD as a enum:COLOR_INTEGRATION_PERIOD
         */
        public static int extractColorIntegrationPeriod(long field) {
            return ((int) field) & 0xf;
        }

        /**
         * Extracts Distance integration period from DistanceIntegrationPeriod.
         * 
         * @param field data bitfield
         * @return DISTANCE_INTEGRATION_PERIOD as a enum:DISTANCE_INTEGRATION_PERIOD
         */
        public static int extractDistanceIntegrationPeriod(long field) {
            return ((int) field) & 0xf;
        }

        /**
         * Extracts Enable digout pad from Digout1OutputConfig.
         * 
         * @param field data bitfield
         * @return output_config as a enum:DIGOUT_OUTPUT_CONFIG
         */
        public static int extractDigout1OutputConfig_OutputConfig(long field) {
            return ((int) field) & 0xff;
        }

        /**
         * Extracts The data source to use in PWM mode. from Digout1OutputConfig.
         * 
         * @param field data bitfield
         * @return pwm_data_source as a enum:DATA_SOURCE
         */
        public static int extractDigout1OutputConfig_PwmDataSource(long field) {
            return ((int) (field >> 8)) & 0xf;
        }

        /**
         * Extracts Enable digout pad from Digout2OutputConfig.
         * 
         * @param field data bitfield
         * @return output_config as a enum:DIGOUT_OUTPUT_CONFIG
         */
        public static int extractDigout2OutputConfig_OutputConfig(long field) {
            return ((int) field) & 0xff;
        }

        /**
         * Extracts The data source to use in PWM mode. from Digout2OutputConfig.
         * 
         * @param field data bitfield
         * @return pwm_data_source as a enum:DATA_SOURCE
         */
        public static int extractDigout2OutputConfig_PwmDataSource(long field) {
            return ((int) (field >> 8)) & 0xf;
        }

        /**
         * Extracts Send digout message on positive edge (false-&gt;true) from Digout1MessageOnChange.
         * 
         * @param field data bitfield
         * @return positive_edge as a bool
         */
        public static boolean extractDigout1MessageOnChange_PositiveEdge(long field) {
            return ((field >> 0) & 1) > 0;
        }

        /**
         * Extracts Send digout message on negative edge (true-&gt;false) from Digout1MessageOnChange.
         * 
         * @param field data bitfield
         * @return negative_edge as a bool
         */
        public static boolean extractDigout1MessageOnChange_NegativeEdge(long field) {
            return ((field >> 1) & 1) > 0;
        }

        /**
         * Extracts Send digout message on positive edge (false-&gt;true) from Digout2MessageOnChange.
         * 
         * @param field data bitfield
         * @return positive_edge as a bool
         */
        public static boolean extractDigout2MessageOnChange_PositiveEdge(long field) {
            return ((field >> 0) & 1) > 0;
        }

        /**
         * Extracts Send digout message on negative edge (true-&gt;false) from Digout2MessageOnChange.
         * 
         * @param field data bitfield
         * @return negative_edge as a bool
         */
        public static boolean extractDigout2MessageOnChange_NegativeEdge(long field) {
            return ((field >> 1) & 1) > 0;
        }

        /**
         * Extracts Enable the digout slot from Digout1Config0.
         * 
         * @param field data bitfield
         * @return slot_enabled as a bool
         */
        public static boolean extractDigout1Config0_SlotEnabled(long field) {
            return ((field >> 0) & 1) > 0;
        }

        /**
         * Extracts How the digout slot interacts with the next slot from Digout1Config0.
         * 
         * @param field data bitfield
         * @return next_slot_action as a enum:NEXT_SLOT_ACTION
         */
        public static int extractDigout1Config0_NextSlotAction(long field) {
            return ((int) (field >> 1)) & 0x3;
        }

        /**
         * Extracts Invert the digout slot's boolean value from Digout1Config0.
         * 
         * @param field data bitfield
         * @return invert_value as a bool
         */
        public static boolean extractDigout1Config0_InvertValue(long field) {
            return ((field >> 3) & 1) > 0;
        }

        /**
         * Extracts Opcode from Digout1Config0.
         * 
         * @param field data bitfield
         * @return opcode as a enum:SLOT_OPCODE
         */
        public static int extractDigout1Config0_Opcode(long field) {
            return ((int) (field >> 4)) & 0x7f;
        }

        /**
         * Extracts Additive immidiate from Digout1Config0.
         * 
         * @param field data bitfield
         * @return immidiate_additive as a sint:21
         */
        public static int extractDigout1Config0_ImmidiateAdditive(long field) {
            return ((((int) (field >> 11)) & 0x1fffff) << 11) >> 11;
        }

        /**
         * Extracts Scaling immidiate from Digout1Config0.
         * 
         * @param field data bitfield
         * @return immidiate_scaling as a uint:8
         */
        public static int extractDigout1Config0_ImmidiateScaling(long field) {
            return ((int) (field >> 32)) & 0xff;
        }

        /**
         * Extracts First ``a`` data source from Digout1Config0.
         * 
         * @param field data bitfield
         * @return data_source_a as a enum:DATA_SOURCE
         */
        public static int extractDigout1Config0_DataSourceA(long field) {
            return ((int) (field >> 40)) & 0xf;
        }

        /**
         * Extracts Second ``b`` data source from Digout1Config0.
         * 
         * @param field data bitfield
         * @return data_source_b as a enum:DATA_SOURCE
         */
        public static int extractDigout1Config0_DataSourceB(long field) {
            return ((int) (field >> 44)) & 0xf;
        }

        /**
         * Extracts Enable the digout slot from Digout1Config1.
         * 
         * @param field data bitfield
         * @return slot_enabled as a bool
         */
        public static boolean extractDigout1Config1_SlotEnabled(long field) {
            return ((field >> 0) & 1) > 0;
        }

        /**
         * Extracts How the digout slot interacts with the next slot from Digout1Config1.
         * 
         * @param field data bitfield
         * @return next_slot_action as a enum:NEXT_SLOT_ACTION
         */
        public static int extractDigout1Config1_NextSlotAction(long field) {
            return ((int) (field >> 1)) & 0x3;
        }

        /**
         * Extracts Invert the digout slot's boolean value from Digout1Config1.
         * 
         * @param field data bitfield
         * @return invert_value as a bool
         */
        public static boolean extractDigout1Config1_InvertValue(long field) {
            return ((field >> 3) & 1) > 0;
        }

        /**
         * Extracts Opcode from Digout1Config1.
         * 
         * @param field data bitfield
         * @return opcode as a enum:SLOT_OPCODE
         */
        public static int extractDigout1Config1_Opcode(long field) {
            return ((int) (field >> 4)) & 0x7f;
        }

        /**
         * Extracts Additive immidiate from Digout1Config1.
         * 
         * @param field data bitfield
         * @return immidiate_additive as a sint:21
         */
        public static int extractDigout1Config1_ImmidiateAdditive(long field) {
            return ((((int) (field >> 11)) & 0x1fffff) << 11) >> 11;
        }

        /**
         * Extracts Scaling immidiate from Digout1Config1.
         * 
         * @param field data bitfield
         * @return immidiate_scaling as a uint:8
         */
        public static int extractDigout1Config1_ImmidiateScaling(long field) {
            return ((int) (field >> 32)) & 0xff;
        }

        /**
         * Extracts First ``a`` data source from Digout1Config1.
         * 
         * @param field data bitfield
         * @return data_source_a as a enum:DATA_SOURCE
         */
        public static int extractDigout1Config1_DataSourceA(long field) {
            return ((int) (field >> 40)) & 0xf;
        }

        /**
         * Extracts Second ``b`` data source from Digout1Config1.
         * 
         * @param field data bitfield
         * @return data_source_b as a enum:DATA_SOURCE
         */
        public static int extractDigout1Config1_DataSourceB(long field) {
            return ((int) (field >> 44)) & 0xf;
        }

        /**
         * Extracts Enable the digout slot from Digout1Config2.
         * 
         * @param field data bitfield
         * @return slot_enabled as a bool
         */
        public static boolean extractDigout1Config2_SlotEnabled(long field) {
            return ((field >> 0) & 1) > 0;
        }

        /**
         * Extracts How the digout slot interacts with the next slot from Digout1Config2.
         * 
         * @param field data bitfield
         * @return next_slot_action as a enum:NEXT_SLOT_ACTION
         */
        public static int extractDigout1Config2_NextSlotAction(long field) {
            return ((int) (field >> 1)) & 0x3;
        }

        /**
         * Extracts Invert the digout slot's boolean value from Digout1Config2.
         * 
         * @param field data bitfield
         * @return invert_value as a bool
         */
        public static boolean extractDigout1Config2_InvertValue(long field) {
            return ((field >> 3) & 1) > 0;
        }

        /**
         * Extracts Opcode from Digout1Config2.
         * 
         * @param field data bitfield
         * @return opcode as a enum:SLOT_OPCODE
         */
        public static int extractDigout1Config2_Opcode(long field) {
            return ((int) (field >> 4)) & 0x7f;
        }

        /**
         * Extracts Additive immidiate from Digout1Config2.
         * 
         * @param field data bitfield
         * @return immidiate_additive as a sint:21
         */
        public static int extractDigout1Config2_ImmidiateAdditive(long field) {
            return ((((int) (field >> 11)) & 0x1fffff) << 11) >> 11;
        }

        /**
         * Extracts Scaling immidiate from Digout1Config2.
         * 
         * @param field data bitfield
         * @return immidiate_scaling as a uint:8
         */
        public static int extractDigout1Config2_ImmidiateScaling(long field) {
            return ((int) (field >> 32)) & 0xff;
        }

        /**
         * Extracts First ``a`` data source from Digout1Config2.
         * 
         * @param field data bitfield
         * @return data_source_a as a enum:DATA_SOURCE
         */
        public static int extractDigout1Config2_DataSourceA(long field) {
            return ((int) (field >> 40)) & 0xf;
        }

        /**
         * Extracts Second ``b`` data source from Digout1Config2.
         * 
         * @param field data bitfield
         * @return data_source_b as a enum:DATA_SOURCE
         */
        public static int extractDigout1Config2_DataSourceB(long field) {
            return ((int) (field >> 44)) & 0xf;
        }

        /**
         * Extracts Enable the digout slot from Digout1Config3.
         * 
         * @param field data bitfield
         * @return slot_enabled as a bool
         */
        public static boolean extractDigout1Config3_SlotEnabled(long field) {
            return ((field >> 0) & 1) > 0;
        }

        /**
         * Extracts How the digout slot interacts with the next slot from Digout1Config3.
         * 
         * @param field data bitfield
         * @return next_slot_action as a enum:NEXT_SLOT_ACTION
         */
        public static int extractDigout1Config3_NextSlotAction(long field) {
            return ((int) (field >> 1)) & 0x3;
        }

        /**
         * Extracts Invert the digout slot's boolean value from Digout1Config3.
         * 
         * @param field data bitfield
         * @return invert_value as a bool
         */
        public static boolean extractDigout1Config3_InvertValue(long field) {
            return ((field >> 3) & 1) > 0;
        }

        /**
         * Extracts Opcode from Digout1Config3.
         * 
         * @param field data bitfield
         * @return opcode as a enum:SLOT_OPCODE
         */
        public static int extractDigout1Config3_Opcode(long field) {
            return ((int) (field >> 4)) & 0x7f;
        }

        /**
         * Extracts Additive immidiate from Digout1Config3.
         * 
         * @param field data bitfield
         * @return immidiate_additive as a sint:21
         */
        public static int extractDigout1Config3_ImmidiateAdditive(long field) {
            return ((((int) (field >> 11)) & 0x1fffff) << 11) >> 11;
        }

        /**
         * Extracts Scaling immidiate from Digout1Config3.
         * 
         * @param field data bitfield
         * @return immidiate_scaling as a uint:8
         */
        public static int extractDigout1Config3_ImmidiateScaling(long field) {
            return ((int) (field >> 32)) & 0xff;
        }

        /**
         * Extracts First ``a`` data source from Digout1Config3.
         * 
         * @param field data bitfield
         * @return data_source_a as a enum:DATA_SOURCE
         */
        public static int extractDigout1Config3_DataSourceA(long field) {
            return ((int) (field >> 40)) & 0xf;
        }

        /**
         * Extracts Second ``b`` data source from Digout1Config3.
         * 
         * @param field data bitfield
         * @return data_source_b as a enum:DATA_SOURCE
         */
        public static int extractDigout1Config3_DataSourceB(long field) {
            return ((int) (field >> 44)) & 0xf;
        }

        /**
         * Extracts Enable the digout slot from Digout1Config4.
         * 
         * @param field data bitfield
         * @return slot_enabled as a bool
         */
        public static boolean extractDigout1Config4_SlotEnabled(long field) {
            return ((field >> 0) & 1) > 0;
        }

        /**
         * Extracts How the digout slot interacts with the next slot from Digout1Config4.
         * 
         * @param field data bitfield
         * @return next_slot_action as a enum:NEXT_SLOT_ACTION
         */
        public static int extractDigout1Config4_NextSlotAction(long field) {
            return ((int) (field >> 1)) & 0x3;
        }

        /**
         * Extracts Invert the digout slot's boolean value from Digout1Config4.
         * 
         * @param field data bitfield
         * @return invert_value as a bool
         */
        public static boolean extractDigout1Config4_InvertValue(long field) {
            return ((field >> 3) & 1) > 0;
        }

        /**
         * Extracts Opcode from Digout1Config4.
         * 
         * @param field data bitfield
         * @return opcode as a enum:SLOT_OPCODE
         */
        public static int extractDigout1Config4_Opcode(long field) {
            return ((int) (field >> 4)) & 0x7f;
        }

        /**
         * Extracts Additive immidiate from Digout1Config4.
         * 
         * @param field data bitfield
         * @return immidiate_additive as a sint:21
         */
        public static int extractDigout1Config4_ImmidiateAdditive(long field) {
            return ((((int) (field >> 11)) & 0x1fffff) << 11) >> 11;
        }

        /**
         * Extracts Scaling immidiate from Digout1Config4.
         * 
         * @param field data bitfield
         * @return immidiate_scaling as a uint:8
         */
        public static int extractDigout1Config4_ImmidiateScaling(long field) {
            return ((int) (field >> 32)) & 0xff;
        }

        /**
         * Extracts First ``a`` data source from Digout1Config4.
         * 
         * @param field data bitfield
         * @return data_source_a as a enum:DATA_SOURCE
         */
        public static int extractDigout1Config4_DataSourceA(long field) {
            return ((int) (field >> 40)) & 0xf;
        }

        /**
         * Extracts Second ``b`` data source from Digout1Config4.
         * 
         * @param field data bitfield
         * @return data_source_b as a enum:DATA_SOURCE
         */
        public static int extractDigout1Config4_DataSourceB(long field) {
            return ((int) (field >> 44)) & 0xf;
        }

        /**
         * Extracts Enable the digout slot from Digout1Config5.
         * 
         * @param field data bitfield
         * @return slot_enabled as a bool
         */
        public static boolean extractDigout1Config5_SlotEnabled(long field) {
            return ((field >> 0) & 1) > 0;
        }

        /**
         * Extracts How the digout slot interacts with the next slot from Digout1Config5.
         * 
         * @param field data bitfield
         * @return next_slot_action as a enum:NEXT_SLOT_ACTION
         */
        public static int extractDigout1Config5_NextSlotAction(long field) {
            return ((int) (field >> 1)) & 0x3;
        }

        /**
         * Extracts Invert the digout slot's boolean value from Digout1Config5.
         * 
         * @param field data bitfield
         * @return invert_value as a bool
         */
        public static boolean extractDigout1Config5_InvertValue(long field) {
            return ((field >> 3) & 1) > 0;
        }

        /**
         * Extracts Opcode from Digout1Config5.
         * 
         * @param field data bitfield
         * @return opcode as a enum:SLOT_OPCODE
         */
        public static int extractDigout1Config5_Opcode(long field) {
            return ((int) (field >> 4)) & 0x7f;
        }

        /**
         * Extracts Additive immidiate from Digout1Config5.
         * 
         * @param field data bitfield
         * @return immidiate_additive as a sint:21
         */
        public static int extractDigout1Config5_ImmidiateAdditive(long field) {
            return ((((int) (field >> 11)) & 0x1fffff) << 11) >> 11;
        }

        /**
         * Extracts Scaling immidiate from Digout1Config5.
         * 
         * @param field data bitfield
         * @return immidiate_scaling as a uint:8
         */
        public static int extractDigout1Config5_ImmidiateScaling(long field) {
            return ((int) (field >> 32)) & 0xff;
        }

        /**
         * Extracts First ``a`` data source from Digout1Config5.
         * 
         * @param field data bitfield
         * @return data_source_a as a enum:DATA_SOURCE
         */
        public static int extractDigout1Config5_DataSourceA(long field) {
            return ((int) (field >> 40)) & 0xf;
        }

        /**
         * Extracts Second ``b`` data source from Digout1Config5.
         * 
         * @param field data bitfield
         * @return data_source_b as a enum:DATA_SOURCE
         */
        public static int extractDigout1Config5_DataSourceB(long field) {
            return ((int) (field >> 44)) & 0xf;
        }

        /**
         * Extracts Enable the digout slot from Digout1Config6.
         * 
         * @param field data bitfield
         * @return slot_enabled as a bool
         */
        public static boolean extractDigout1Config6_SlotEnabled(long field) {
            return ((field >> 0) & 1) > 0;
        }

        /**
         * Extracts How the digout slot interacts with the next slot from Digout1Config6.
         * 
         * @param field data bitfield
         * @return next_slot_action as a enum:NEXT_SLOT_ACTION
         */
        public static int extractDigout1Config6_NextSlotAction(long field) {
            return ((int) (field >> 1)) & 0x3;
        }

        /**
         * Extracts Invert the digout slot's boolean value from Digout1Config6.
         * 
         * @param field data bitfield
         * @return invert_value as a bool
         */
        public static boolean extractDigout1Config6_InvertValue(long field) {
            return ((field >> 3) & 1) > 0;
        }

        /**
         * Extracts Opcode from Digout1Config6.
         * 
         * @param field data bitfield
         * @return opcode as a enum:SLOT_OPCODE
         */
        public static int extractDigout1Config6_Opcode(long field) {
            return ((int) (field >> 4)) & 0x7f;
        }

        /**
         * Extracts Additive immidiate from Digout1Config6.
         * 
         * @param field data bitfield
         * @return immidiate_additive as a sint:21
         */
        public static int extractDigout1Config6_ImmidiateAdditive(long field) {
            return ((((int) (field >> 11)) & 0x1fffff) << 11) >> 11;
        }

        /**
         * Extracts Scaling immidiate from Digout1Config6.
         * 
         * @param field data bitfield
         * @return immidiate_scaling as a uint:8
         */
        public static int extractDigout1Config6_ImmidiateScaling(long field) {
            return ((int) (field >> 32)) & 0xff;
        }

        /**
         * Extracts First ``a`` data source from Digout1Config6.
         * 
         * @param field data bitfield
         * @return data_source_a as a enum:DATA_SOURCE
         */
        public static int extractDigout1Config6_DataSourceA(long field) {
            return ((int) (field >> 40)) & 0xf;
        }

        /**
         * Extracts Second ``b`` data source from Digout1Config6.
         * 
         * @param field data bitfield
         * @return data_source_b as a enum:DATA_SOURCE
         */
        public static int extractDigout1Config6_DataSourceB(long field) {
            return ((int) (field >> 44)) & 0xf;
        }

        /**
         * Extracts Enable the digout slot from Digout1Config7.
         * 
         * @param field data bitfield
         * @return slot_enabled as a bool
         */
        public static boolean extractDigout1Config7_SlotEnabled(long field) {
            return ((field >> 0) & 1) > 0;
        }

        /**
         * Extracts How the digout slot interacts with the next slot from Digout1Config7.
         * 
         * @param field data bitfield
         * @return next_slot_action as a enum:NEXT_SLOT_ACTION
         */
        public static int extractDigout1Config7_NextSlotAction(long field) {
            return ((int) (field >> 1)) & 0x3;
        }

        /**
         * Extracts Invert the digout slot's boolean value from Digout1Config7.
         * 
         * @param field data bitfield
         * @return invert_value as a bool
         */
        public static boolean extractDigout1Config7_InvertValue(long field) {
            return ((field >> 3) & 1) > 0;
        }

        /**
         * Extracts Opcode from Digout1Config7.
         * 
         * @param field data bitfield
         * @return opcode as a enum:SLOT_OPCODE
         */
        public static int extractDigout1Config7_Opcode(long field) {
            return ((int) (field >> 4)) & 0x7f;
        }

        /**
         * Extracts Additive immidiate from Digout1Config7.
         * 
         * @param field data bitfield
         * @return immidiate_additive as a sint:21
         */
        public static int extractDigout1Config7_ImmidiateAdditive(long field) {
            return ((((int) (field >> 11)) & 0x1fffff) << 11) >> 11;
        }

        /**
         * Extracts Scaling immidiate from Digout1Config7.
         * 
         * @param field data bitfield
         * @return immidiate_scaling as a uint:8
         */
        public static int extractDigout1Config7_ImmidiateScaling(long field) {
            return ((int) (field >> 32)) & 0xff;
        }

        /**
         * Extracts First ``a`` data source from Digout1Config7.
         * 
         * @param field data bitfield
         * @return data_source_a as a enum:DATA_SOURCE
         */
        public static int extractDigout1Config7_DataSourceA(long field) {
            return ((int) (field >> 40)) & 0xf;
        }

        /**
         * Extracts Second ``b`` data source from Digout1Config7.
         * 
         * @param field data bitfield
         * @return data_source_b as a enum:DATA_SOURCE
         */
        public static int extractDigout1Config7_DataSourceB(long field) {
            return ((int) (field >> 44)) & 0xf;
        }

        /**
         * Extracts Enable the digout slot from Digout1Config8.
         * 
         * @param field data bitfield
         * @return slot_enabled as a bool
         */
        public static boolean extractDigout1Config8_SlotEnabled(long field) {
            return ((field >> 0) & 1) > 0;
        }

        /**
         * Extracts How the digout slot interacts with the next slot from Digout1Config8.
         * 
         * @param field data bitfield
         * @return next_slot_action as a enum:NEXT_SLOT_ACTION
         */
        public static int extractDigout1Config8_NextSlotAction(long field) {
            return ((int) (field >> 1)) & 0x3;
        }

        /**
         * Extracts Invert the digout slot's boolean value from Digout1Config8.
         * 
         * @param field data bitfield
         * @return invert_value as a bool
         */
        public static boolean extractDigout1Config8_InvertValue(long field) {
            return ((field >> 3) & 1) > 0;
        }

        /**
         * Extracts Opcode from Digout1Config8.
         * 
         * @param field data bitfield
         * @return opcode as a enum:SLOT_OPCODE
         */
        public static int extractDigout1Config8_Opcode(long field) {
            return ((int) (field >> 4)) & 0x7f;
        }

        /**
         * Extracts Additive immidiate from Digout1Config8.
         * 
         * @param field data bitfield
         * @return immidiate_additive as a sint:21
         */
        public static int extractDigout1Config8_ImmidiateAdditive(long field) {
            return ((((int) (field >> 11)) & 0x1fffff) << 11) >> 11;
        }

        /**
         * Extracts Scaling immidiate from Digout1Config8.
         * 
         * @param field data bitfield
         * @return immidiate_scaling as a uint:8
         */
        public static int extractDigout1Config8_ImmidiateScaling(long field) {
            return ((int) (field >> 32)) & 0xff;
        }

        /**
         * Extracts First ``a`` data source from Digout1Config8.
         * 
         * @param field data bitfield
         * @return data_source_a as a enum:DATA_SOURCE
         */
        public static int extractDigout1Config8_DataSourceA(long field) {
            return ((int) (field >> 40)) & 0xf;
        }

        /**
         * Extracts Second ``b`` data source from Digout1Config8.
         * 
         * @param field data bitfield
         * @return data_source_b as a enum:DATA_SOURCE
         */
        public static int extractDigout1Config8_DataSourceB(long field) {
            return ((int) (field >> 44)) & 0xf;
        }

        /**
         * Extracts Enable the digout slot from Digout1Config9.
         * 
         * @param field data bitfield
         * @return slot_enabled as a bool
         */
        public static boolean extractDigout1Config9_SlotEnabled(long field) {
            return ((field >> 0) & 1) > 0;
        }

        /**
         * Extracts How the digout slot interacts with the next slot from Digout1Config9.
         * 
         * @param field data bitfield
         * @return next_slot_action as a enum:NEXT_SLOT_ACTION
         */
        public static int extractDigout1Config9_NextSlotAction(long field) {
            return ((int) (field >> 1)) & 0x3;
        }

        /**
         * Extracts Invert the digout slot's boolean value from Digout1Config9.
         * 
         * @param field data bitfield
         * @return invert_value as a bool
         */
        public static boolean extractDigout1Config9_InvertValue(long field) {
            return ((field >> 3) & 1) > 0;
        }

        /**
         * Extracts Opcode from Digout1Config9.
         * 
         * @param field data bitfield
         * @return opcode as a enum:SLOT_OPCODE
         */
        public static int extractDigout1Config9_Opcode(long field) {
            return ((int) (field >> 4)) & 0x7f;
        }

        /**
         * Extracts Additive immidiate from Digout1Config9.
         * 
         * @param field data bitfield
         * @return immidiate_additive as a sint:21
         */
        public static int extractDigout1Config9_ImmidiateAdditive(long field) {
            return ((((int) (field >> 11)) & 0x1fffff) << 11) >> 11;
        }

        /**
         * Extracts Scaling immidiate from Digout1Config9.
         * 
         * @param field data bitfield
         * @return immidiate_scaling as a uint:8
         */
        public static int extractDigout1Config9_ImmidiateScaling(long field) {
            return ((int) (field >> 32)) & 0xff;
        }

        /**
         * Extracts First ``a`` data source from Digout1Config9.
         * 
         * @param field data bitfield
         * @return data_source_a as a enum:DATA_SOURCE
         */
        public static int extractDigout1Config9_DataSourceA(long field) {
            return ((int) (field >> 40)) & 0xf;
        }

        /**
         * Extracts Second ``b`` data source from Digout1Config9.
         * 
         * @param field data bitfield
         * @return data_source_b as a enum:DATA_SOURCE
         */
        public static int extractDigout1Config9_DataSourceB(long field) {
            return ((int) (field >> 44)) & 0xf;
        }

        /**
         * Extracts Enable the digout slot from Digout1Config10.
         * 
         * @param field data bitfield
         * @return slot_enabled as a bool
         */
        public static boolean extractDigout1Config10_SlotEnabled(long field) {
            return ((field >> 0) & 1) > 0;
        }

        /**
         * Extracts How the digout slot interacts with the next slot from Digout1Config10.
         * 
         * @param field data bitfield
         * @return next_slot_action as a enum:NEXT_SLOT_ACTION
         */
        public static int extractDigout1Config10_NextSlotAction(long field) {
            return ((int) (field >> 1)) & 0x3;
        }

        /**
         * Extracts Invert the digout slot's boolean value from Digout1Config10.
         * 
         * @param field data bitfield
         * @return invert_value as a bool
         */
        public static boolean extractDigout1Config10_InvertValue(long field) {
            return ((field >> 3) & 1) > 0;
        }

        /**
         * Extracts Opcode from Digout1Config10.
         * 
         * @param field data bitfield
         * @return opcode as a enum:SLOT_OPCODE
         */
        public static int extractDigout1Config10_Opcode(long field) {
            return ((int) (field >> 4)) & 0x7f;
        }

        /**
         * Extracts Additive immidiate from Digout1Config10.
         * 
         * @param field data bitfield
         * @return immidiate_additive as a sint:21
         */
        public static int extractDigout1Config10_ImmidiateAdditive(long field) {
            return ((((int) (field >> 11)) & 0x1fffff) << 11) >> 11;
        }

        /**
         * Extracts Scaling immidiate from Digout1Config10.
         * 
         * @param field data bitfield
         * @return immidiate_scaling as a uint:8
         */
        public static int extractDigout1Config10_ImmidiateScaling(long field) {
            return ((int) (field >> 32)) & 0xff;
        }

        /**
         * Extracts First ``a`` data source from Digout1Config10.
         * 
         * @param field data bitfield
         * @return data_source_a as a enum:DATA_SOURCE
         */
        public static int extractDigout1Config10_DataSourceA(long field) {
            return ((int) (field >> 40)) & 0xf;
        }

        /**
         * Extracts Second ``b`` data source from Digout1Config10.
         * 
         * @param field data bitfield
         * @return data_source_b as a enum:DATA_SOURCE
         */
        public static int extractDigout1Config10_DataSourceB(long field) {
            return ((int) (field >> 44)) & 0xf;
        }

        /**
         * Extracts Enable the digout slot from Digout1Config11.
         * 
         * @param field data bitfield
         * @return slot_enabled as a bool
         */
        public static boolean extractDigout1Config11_SlotEnabled(long field) {
            return ((field >> 0) & 1) > 0;
        }

        /**
         * Extracts How the digout slot interacts with the next slot from Digout1Config11.
         * 
         * @param field data bitfield
         * @return next_slot_action as a enum:NEXT_SLOT_ACTION
         */
        public static int extractDigout1Config11_NextSlotAction(long field) {
            return ((int) (field >> 1)) & 0x3;
        }

        /**
         * Extracts Invert the digout slot's boolean value from Digout1Config11.
         * 
         * @param field data bitfield
         * @return invert_value as a bool
         */
        public static boolean extractDigout1Config11_InvertValue(long field) {
            return ((field >> 3) & 1) > 0;
        }

        /**
         * Extracts Opcode from Digout1Config11.
         * 
         * @param field data bitfield
         * @return opcode as a enum:SLOT_OPCODE
         */
        public static int extractDigout1Config11_Opcode(long field) {
            return ((int) (field >> 4)) & 0x7f;
        }

        /**
         * Extracts Additive immidiate from Digout1Config11.
         * 
         * @param field data bitfield
         * @return immidiate_additive as a sint:21
         */
        public static int extractDigout1Config11_ImmidiateAdditive(long field) {
            return ((((int) (field >> 11)) & 0x1fffff) << 11) >> 11;
        }

        /**
         * Extracts Scaling immidiate from Digout1Config11.
         * 
         * @param field data bitfield
         * @return immidiate_scaling as a uint:8
         */
        public static int extractDigout1Config11_ImmidiateScaling(long field) {
            return ((int) (field >> 32)) & 0xff;
        }

        /**
         * Extracts First ``a`` data source from Digout1Config11.
         * 
         * @param field data bitfield
         * @return data_source_a as a enum:DATA_SOURCE
         */
        public static int extractDigout1Config11_DataSourceA(long field) {
            return ((int) (field >> 40)) & 0xf;
        }

        /**
         * Extracts Second ``b`` data source from Digout1Config11.
         * 
         * @param field data bitfield
         * @return data_source_b as a enum:DATA_SOURCE
         */
        public static int extractDigout1Config11_DataSourceB(long field) {
            return ((int) (field >> 44)) & 0xf;
        }

        /**
         * Extracts Enable the digout slot from Digout1Config12.
         * 
         * @param field data bitfield
         * @return slot_enabled as a bool
         */
        public static boolean extractDigout1Config12_SlotEnabled(long field) {
            return ((field >> 0) & 1) > 0;
        }

        /**
         * Extracts How the digout slot interacts with the next slot from Digout1Config12.
         * 
         * @param field data bitfield
         * @return next_slot_action as a enum:NEXT_SLOT_ACTION
         */
        public static int extractDigout1Config12_NextSlotAction(long field) {
            return ((int) (field >> 1)) & 0x3;
        }

        /**
         * Extracts Invert the digout slot's boolean value from Digout1Config12.
         * 
         * @param field data bitfield
         * @return invert_value as a bool
         */
        public static boolean extractDigout1Config12_InvertValue(long field) {
            return ((field >> 3) & 1) > 0;
        }

        /**
         * Extracts Opcode from Digout1Config12.
         * 
         * @param field data bitfield
         * @return opcode as a enum:SLOT_OPCODE
         */
        public static int extractDigout1Config12_Opcode(long field) {
            return ((int) (field >> 4)) & 0x7f;
        }

        /**
         * Extracts Additive immidiate from Digout1Config12.
         * 
         * @param field data bitfield
         * @return immidiate_additive as a sint:21
         */
        public static int extractDigout1Config12_ImmidiateAdditive(long field) {
            return ((((int) (field >> 11)) & 0x1fffff) << 11) >> 11;
        }

        /**
         * Extracts Scaling immidiate from Digout1Config12.
         * 
         * @param field data bitfield
         * @return immidiate_scaling as a uint:8
         */
        public static int extractDigout1Config12_ImmidiateScaling(long field) {
            return ((int) (field >> 32)) & 0xff;
        }

        /**
         * Extracts First ``a`` data source from Digout1Config12.
         * 
         * @param field data bitfield
         * @return data_source_a as a enum:DATA_SOURCE
         */
        public static int extractDigout1Config12_DataSourceA(long field) {
            return ((int) (field >> 40)) & 0xf;
        }

        /**
         * Extracts Second ``b`` data source from Digout1Config12.
         * 
         * @param field data bitfield
         * @return data_source_b as a enum:DATA_SOURCE
         */
        public static int extractDigout1Config12_DataSourceB(long field) {
            return ((int) (field >> 44)) & 0xf;
        }

        /**
         * Extracts Enable the digout slot from Digout1Config13.
         * 
         * @param field data bitfield
         * @return slot_enabled as a bool
         */
        public static boolean extractDigout1Config13_SlotEnabled(long field) {
            return ((field >> 0) & 1) > 0;
        }

        /**
         * Extracts How the digout slot interacts with the next slot from Digout1Config13.
         * 
         * @param field data bitfield
         * @return next_slot_action as a enum:NEXT_SLOT_ACTION
         */
        public static int extractDigout1Config13_NextSlotAction(long field) {
            return ((int) (field >> 1)) & 0x3;
        }

        /**
         * Extracts Invert the digout slot's boolean value from Digout1Config13.
         * 
         * @param field data bitfield
         * @return invert_value as a bool
         */
        public static boolean extractDigout1Config13_InvertValue(long field) {
            return ((field >> 3) & 1) > 0;
        }

        /**
         * Extracts Opcode from Digout1Config13.
         * 
         * @param field data bitfield
         * @return opcode as a enum:SLOT_OPCODE
         */
        public static int extractDigout1Config13_Opcode(long field) {
            return ((int) (field >> 4)) & 0x7f;
        }

        /**
         * Extracts Additive immidiate from Digout1Config13.
         * 
         * @param field data bitfield
         * @return immidiate_additive as a sint:21
         */
        public static int extractDigout1Config13_ImmidiateAdditive(long field) {
            return ((((int) (field >> 11)) & 0x1fffff) << 11) >> 11;
        }

        /**
         * Extracts Scaling immidiate from Digout1Config13.
         * 
         * @param field data bitfield
         * @return immidiate_scaling as a uint:8
         */
        public static int extractDigout1Config13_ImmidiateScaling(long field) {
            return ((int) (field >> 32)) & 0xff;
        }

        /**
         * Extracts First ``a`` data source from Digout1Config13.
         * 
         * @param field data bitfield
         * @return data_source_a as a enum:DATA_SOURCE
         */
        public static int extractDigout1Config13_DataSourceA(long field) {
            return ((int) (field >> 40)) & 0xf;
        }

        /**
         * Extracts Second ``b`` data source from Digout1Config13.
         * 
         * @param field data bitfield
         * @return data_source_b as a enum:DATA_SOURCE
         */
        public static int extractDigout1Config13_DataSourceB(long field) {
            return ((int) (field >> 44)) & 0xf;
        }

        /**
         * Extracts Enable the digout slot from Digout1Config14.
         * 
         * @param field data bitfield
         * @return slot_enabled as a bool
         */
        public static boolean extractDigout1Config14_SlotEnabled(long field) {
            return ((field >> 0) & 1) > 0;
        }

        /**
         * Extracts How the digout slot interacts with the next slot from Digout1Config14.
         * 
         * @param field data bitfield
         * @return next_slot_action as a enum:NEXT_SLOT_ACTION
         */
        public static int extractDigout1Config14_NextSlotAction(long field) {
            return ((int) (field >> 1)) & 0x3;
        }

        /**
         * Extracts Invert the digout slot's boolean value from Digout1Config14.
         * 
         * @param field data bitfield
         * @return invert_value as a bool
         */
        public static boolean extractDigout1Config14_InvertValue(long field) {
            return ((field >> 3) & 1) > 0;
        }

        /**
         * Extracts Opcode from Digout1Config14.
         * 
         * @param field data bitfield
         * @return opcode as a enum:SLOT_OPCODE
         */
        public static int extractDigout1Config14_Opcode(long field) {
            return ((int) (field >> 4)) & 0x7f;
        }

        /**
         * Extracts Additive immidiate from Digout1Config14.
         * 
         * @param field data bitfield
         * @return immidiate_additive as a sint:21
         */
        public static int extractDigout1Config14_ImmidiateAdditive(long field) {
            return ((((int) (field >> 11)) & 0x1fffff) << 11) >> 11;
        }

        /**
         * Extracts Scaling immidiate from Digout1Config14.
         * 
         * @param field data bitfield
         * @return immidiate_scaling as a uint:8
         */
        public static int extractDigout1Config14_ImmidiateScaling(long field) {
            return ((int) (field >> 32)) & 0xff;
        }

        /**
         * Extracts First ``a`` data source from Digout1Config14.
         * 
         * @param field data bitfield
         * @return data_source_a as a enum:DATA_SOURCE
         */
        public static int extractDigout1Config14_DataSourceA(long field) {
            return ((int) (field >> 40)) & 0xf;
        }

        /**
         * Extracts Second ``b`` data source from Digout1Config14.
         * 
         * @param field data bitfield
         * @return data_source_b as a enum:DATA_SOURCE
         */
        public static int extractDigout1Config14_DataSourceB(long field) {
            return ((int) (field >> 44)) & 0xf;
        }

        /**
         * Extracts Enable the digout slot from Digout1Config15.
         * 
         * @param field data bitfield
         * @return slot_enabled as a bool
         */
        public static boolean extractDigout1Config15_SlotEnabled(long field) {
            return ((field >> 0) & 1) > 0;
        }

        /**
         * Extracts How the digout slot interacts with the next slot from Digout1Config15.
         * 
         * @param field data bitfield
         * @return next_slot_action as a enum:NEXT_SLOT_ACTION
         */
        public static int extractDigout1Config15_NextSlotAction(long field) {
            return ((int) (field >> 1)) & 0x3;
        }

        /**
         * Extracts Invert the digout slot's boolean value from Digout1Config15.
         * 
         * @param field data bitfield
         * @return invert_value as a bool
         */
        public static boolean extractDigout1Config15_InvertValue(long field) {
            return ((field >> 3) & 1) > 0;
        }

        /**
         * Extracts Opcode from Digout1Config15.
         * 
         * @param field data bitfield
         * @return opcode as a enum:SLOT_OPCODE
         */
        public static int extractDigout1Config15_Opcode(long field) {
            return ((int) (field >> 4)) & 0x7f;
        }

        /**
         * Extracts Additive immidiate from Digout1Config15.
         * 
         * @param field data bitfield
         * @return immidiate_additive as a sint:21
         */
        public static int extractDigout1Config15_ImmidiateAdditive(long field) {
            return ((((int) (field >> 11)) & 0x1fffff) << 11) >> 11;
        }

        /**
         * Extracts Scaling immidiate from Digout1Config15.
         * 
         * @param field data bitfield
         * @return immidiate_scaling as a uint:8
         */
        public static int extractDigout1Config15_ImmidiateScaling(long field) {
            return ((int) (field >> 32)) & 0xff;
        }

        /**
         * Extracts First ``a`` data source from Digout1Config15.
         * 
         * @param field data bitfield
         * @return data_source_a as a enum:DATA_SOURCE
         */
        public static int extractDigout1Config15_DataSourceA(long field) {
            return ((int) (field >> 40)) & 0xf;
        }

        /**
         * Extracts Second ``b`` data source from Digout1Config15.
         * 
         * @param field data bitfield
         * @return data_source_b as a enum:DATA_SOURCE
         */
        public static int extractDigout1Config15_DataSourceB(long field) {
            return ((int) (field >> 44)) & 0xf;
        }

        /**
         * Extracts Enable the digout slot from Digout2Config0.
         * 
         * @param field data bitfield
         * @return slot_enabled as a bool
         */
        public static boolean extractDigout2Config0_SlotEnabled(long field) {
            return ((field >> 0) & 1) > 0;
        }

        /**
         * Extracts How the digout slot interacts with the next slot from Digout2Config0.
         * 
         * @param field data bitfield
         * @return next_slot_action as a enum:NEXT_SLOT_ACTION
         */
        public static int extractDigout2Config0_NextSlotAction(long field) {
            return ((int) (field >> 1)) & 0x3;
        }

        /**
         * Extracts Invert the digout slot's boolean value from Digout2Config0.
         * 
         * @param field data bitfield
         * @return invert_value as a bool
         */
        public static boolean extractDigout2Config0_InvertValue(long field) {
            return ((field >> 3) & 1) > 0;
        }

        /**
         * Extracts Opcode from Digout2Config0.
         * 
         * @param field data bitfield
         * @return opcode as a enum:SLOT_OPCODE
         */
        public static int extractDigout2Config0_Opcode(long field) {
            return ((int) (field >> 4)) & 0x7f;
        }

        /**
         * Extracts Additive immidiate from Digout2Config0.
         * 
         * @param field data bitfield
         * @return immidiate_additive as a sint:21
         */
        public static int extractDigout2Config0_ImmidiateAdditive(long field) {
            return ((((int) (field >> 11)) & 0x1fffff) << 11) >> 11;
        }

        /**
         * Extracts Scaling immidiate from Digout2Config0.
         * 
         * @param field data bitfield
         * @return immidiate_scaling as a uint:8
         */
        public static int extractDigout2Config0_ImmidiateScaling(long field) {
            return ((int) (field >> 32)) & 0xff;
        }

        /**
         * Extracts First ``a`` data source from Digout2Config0.
         * 
         * @param field data bitfield
         * @return data_source_a as a enum:DATA_SOURCE
         */
        public static int extractDigout2Config0_DataSourceA(long field) {
            return ((int) (field >> 40)) & 0xf;
        }

        /**
         * Extracts Second ``b`` data source from Digout2Config0.
         * 
         * @param field data bitfield
         * @return data_source_b as a enum:DATA_SOURCE
         */
        public static int extractDigout2Config0_DataSourceB(long field) {
            return ((int) (field >> 44)) & 0xf;
        }

        /**
         * Extracts Enable the digout slot from Digout2Config1.
         * 
         * @param field data bitfield
         * @return slot_enabled as a bool
         */
        public static boolean extractDigout2Config1_SlotEnabled(long field) {
            return ((field >> 0) & 1) > 0;
        }

        /**
         * Extracts How the digout slot interacts with the next slot from Digout2Config1.
         * 
         * @param field data bitfield
         * @return next_slot_action as a enum:NEXT_SLOT_ACTION
         */
        public static int extractDigout2Config1_NextSlotAction(long field) {
            return ((int) (field >> 1)) & 0x3;
        }

        /**
         * Extracts Invert the digout slot's boolean value from Digout2Config1.
         * 
         * @param field data bitfield
         * @return invert_value as a bool
         */
        public static boolean extractDigout2Config1_InvertValue(long field) {
            return ((field >> 3) & 1) > 0;
        }

        /**
         * Extracts Opcode from Digout2Config1.
         * 
         * @param field data bitfield
         * @return opcode as a enum:SLOT_OPCODE
         */
        public static int extractDigout2Config1_Opcode(long field) {
            return ((int) (field >> 4)) & 0x7f;
        }

        /**
         * Extracts Additive immidiate from Digout2Config1.
         * 
         * @param field data bitfield
         * @return immidiate_additive as a sint:21
         */
        public static int extractDigout2Config1_ImmidiateAdditive(long field) {
            return ((((int) (field >> 11)) & 0x1fffff) << 11) >> 11;
        }

        /**
         * Extracts Scaling immidiate from Digout2Config1.
         * 
         * @param field data bitfield
         * @return immidiate_scaling as a uint:8
         */
        public static int extractDigout2Config1_ImmidiateScaling(long field) {
            return ((int) (field >> 32)) & 0xff;
        }

        /**
         * Extracts First ``a`` data source from Digout2Config1.
         * 
         * @param field data bitfield
         * @return data_source_a as a enum:DATA_SOURCE
         */
        public static int extractDigout2Config1_DataSourceA(long field) {
            return ((int) (field >> 40)) & 0xf;
        }

        /**
         * Extracts Second ``b`` data source from Digout2Config1.
         * 
         * @param field data bitfield
         * @return data_source_b as a enum:DATA_SOURCE
         */
        public static int extractDigout2Config1_DataSourceB(long field) {
            return ((int) (field >> 44)) & 0xf;
        }

        /**
         * Extracts Enable the digout slot from Digout2Config2.
         * 
         * @param field data bitfield
         * @return slot_enabled as a bool
         */
        public static boolean extractDigout2Config2_SlotEnabled(long field) {
            return ((field >> 0) & 1) > 0;
        }

        /**
         * Extracts How the digout slot interacts with the next slot from Digout2Config2.
         * 
         * @param field data bitfield
         * @return next_slot_action as a enum:NEXT_SLOT_ACTION
         */
        public static int extractDigout2Config2_NextSlotAction(long field) {
            return ((int) (field >> 1)) & 0x3;
        }

        /**
         * Extracts Invert the digout slot's boolean value from Digout2Config2.
         * 
         * @param field data bitfield
         * @return invert_value as a bool
         */
        public static boolean extractDigout2Config2_InvertValue(long field) {
            return ((field >> 3) & 1) > 0;
        }

        /**
         * Extracts Opcode from Digout2Config2.
         * 
         * @param field data bitfield
         * @return opcode as a enum:SLOT_OPCODE
         */
        public static int extractDigout2Config2_Opcode(long field) {
            return ((int) (field >> 4)) & 0x7f;
        }

        /**
         * Extracts Additive immidiate from Digout2Config2.
         * 
         * @param field data bitfield
         * @return immidiate_additive as a sint:21
         */
        public static int extractDigout2Config2_ImmidiateAdditive(long field) {
            return ((((int) (field >> 11)) & 0x1fffff) << 11) >> 11;
        }

        /**
         * Extracts Scaling immidiate from Digout2Config2.
         * 
         * @param field data bitfield
         * @return immidiate_scaling as a uint:8
         */
        public static int extractDigout2Config2_ImmidiateScaling(long field) {
            return ((int) (field >> 32)) & 0xff;
        }

        /**
         * Extracts First ``a`` data source from Digout2Config2.
         * 
         * @param field data bitfield
         * @return data_source_a as a enum:DATA_SOURCE
         */
        public static int extractDigout2Config2_DataSourceA(long field) {
            return ((int) (field >> 40)) & 0xf;
        }

        /**
         * Extracts Second ``b`` data source from Digout2Config2.
         * 
         * @param field data bitfield
         * @return data_source_b as a enum:DATA_SOURCE
         */
        public static int extractDigout2Config2_DataSourceB(long field) {
            return ((int) (field >> 44)) & 0xf;
        }

        /**
         * Extracts Enable the digout slot from Digout2Config3.
         * 
         * @param field data bitfield
         * @return slot_enabled as a bool
         */
        public static boolean extractDigout2Config3_SlotEnabled(long field) {
            return ((field >> 0) & 1) > 0;
        }

        /**
         * Extracts How the digout slot interacts with the next slot from Digout2Config3.
         * 
         * @param field data bitfield
         * @return next_slot_action as a enum:NEXT_SLOT_ACTION
         */
        public static int extractDigout2Config3_NextSlotAction(long field) {
            return ((int) (field >> 1)) & 0x3;
        }

        /**
         * Extracts Invert the digout slot's boolean value from Digout2Config3.
         * 
         * @param field data bitfield
         * @return invert_value as a bool
         */
        public static boolean extractDigout2Config3_InvertValue(long field) {
            return ((field >> 3) & 1) > 0;
        }

        /**
         * Extracts Opcode from Digout2Config3.
         * 
         * @param field data bitfield
         * @return opcode as a enum:SLOT_OPCODE
         */
        public static int extractDigout2Config3_Opcode(long field) {
            return ((int) (field >> 4)) & 0x7f;
        }

        /**
         * Extracts Additive immidiate from Digout2Config3.
         * 
         * @param field data bitfield
         * @return immidiate_additive as a sint:21
         */
        public static int extractDigout2Config3_ImmidiateAdditive(long field) {
            return ((((int) (field >> 11)) & 0x1fffff) << 11) >> 11;
        }

        /**
         * Extracts Scaling immidiate from Digout2Config3.
         * 
         * @param field data bitfield
         * @return immidiate_scaling as a uint:8
         */
        public static int extractDigout2Config3_ImmidiateScaling(long field) {
            return ((int) (field >> 32)) & 0xff;
        }

        /**
         * Extracts First ``a`` data source from Digout2Config3.
         * 
         * @param field data bitfield
         * @return data_source_a as a enum:DATA_SOURCE
         */
        public static int extractDigout2Config3_DataSourceA(long field) {
            return ((int) (field >> 40)) & 0xf;
        }

        /**
         * Extracts Second ``b`` data source from Digout2Config3.
         * 
         * @param field data bitfield
         * @return data_source_b as a enum:DATA_SOURCE
         */
        public static int extractDigout2Config3_DataSourceB(long field) {
            return ((int) (field >> 44)) & 0xf;
        }

        /**
         * Extracts Enable the digout slot from Digout2Config4.
         * 
         * @param field data bitfield
         * @return slot_enabled as a bool
         */
        public static boolean extractDigout2Config4_SlotEnabled(long field) {
            return ((field >> 0) & 1) > 0;
        }

        /**
         * Extracts How the digout slot interacts with the next slot from Digout2Config4.
         * 
         * @param field data bitfield
         * @return next_slot_action as a enum:NEXT_SLOT_ACTION
         */
        public static int extractDigout2Config4_NextSlotAction(long field) {
            return ((int) (field >> 1)) & 0x3;
        }

        /**
         * Extracts Invert the digout slot's boolean value from Digout2Config4.
         * 
         * @param field data bitfield
         * @return invert_value as a bool
         */
        public static boolean extractDigout2Config4_InvertValue(long field) {
            return ((field >> 3) & 1) > 0;
        }

        /**
         * Extracts Opcode from Digout2Config4.
         * 
         * @param field data bitfield
         * @return opcode as a enum:SLOT_OPCODE
         */
        public static int extractDigout2Config4_Opcode(long field) {
            return ((int) (field >> 4)) & 0x7f;
        }

        /**
         * Extracts Additive immidiate from Digout2Config4.
         * 
         * @param field data bitfield
         * @return immidiate_additive as a sint:21
         */
        public static int extractDigout2Config4_ImmidiateAdditive(long field) {
            return ((((int) (field >> 11)) & 0x1fffff) << 11) >> 11;
        }

        /**
         * Extracts Scaling immidiate from Digout2Config4.
         * 
         * @param field data bitfield
         * @return immidiate_scaling as a uint:8
         */
        public static int extractDigout2Config4_ImmidiateScaling(long field) {
            return ((int) (field >> 32)) & 0xff;
        }

        /**
         * Extracts First ``a`` data source from Digout2Config4.
         * 
         * @param field data bitfield
         * @return data_source_a as a enum:DATA_SOURCE
         */
        public static int extractDigout2Config4_DataSourceA(long field) {
            return ((int) (field >> 40)) & 0xf;
        }

        /**
         * Extracts Second ``b`` data source from Digout2Config4.
         * 
         * @param field data bitfield
         * @return data_source_b as a enum:DATA_SOURCE
         */
        public static int extractDigout2Config4_DataSourceB(long field) {
            return ((int) (field >> 44)) & 0xf;
        }

        /**
         * Extracts Enable the digout slot from Digout2Config5.
         * 
         * @param field data bitfield
         * @return slot_enabled as a bool
         */
        public static boolean extractDigout2Config5_SlotEnabled(long field) {
            return ((field >> 0) & 1) > 0;
        }

        /**
         * Extracts How the digout slot interacts with the next slot from Digout2Config5.
         * 
         * @param field data bitfield
         * @return next_slot_action as a enum:NEXT_SLOT_ACTION
         */
        public static int extractDigout2Config5_NextSlotAction(long field) {
            return ((int) (field >> 1)) & 0x3;
        }

        /**
         * Extracts Invert the digout slot's boolean value from Digout2Config5.
         * 
         * @param field data bitfield
         * @return invert_value as a bool
         */
        public static boolean extractDigout2Config5_InvertValue(long field) {
            return ((field >> 3) & 1) > 0;
        }

        /**
         * Extracts Opcode from Digout2Config5.
         * 
         * @param field data bitfield
         * @return opcode as a enum:SLOT_OPCODE
         */
        public static int extractDigout2Config5_Opcode(long field) {
            return ((int) (field >> 4)) & 0x7f;
        }

        /**
         * Extracts Additive immidiate from Digout2Config5.
         * 
         * @param field data bitfield
         * @return immidiate_additive as a sint:21
         */
        public static int extractDigout2Config5_ImmidiateAdditive(long field) {
            return ((((int) (field >> 11)) & 0x1fffff) << 11) >> 11;
        }

        /**
         * Extracts Scaling immidiate from Digout2Config5.
         * 
         * @param field data bitfield
         * @return immidiate_scaling as a uint:8
         */
        public static int extractDigout2Config5_ImmidiateScaling(long field) {
            return ((int) (field >> 32)) & 0xff;
        }

        /**
         * Extracts First ``a`` data source from Digout2Config5.
         * 
         * @param field data bitfield
         * @return data_source_a as a enum:DATA_SOURCE
         */
        public static int extractDigout2Config5_DataSourceA(long field) {
            return ((int) (field >> 40)) & 0xf;
        }

        /**
         * Extracts Second ``b`` data source from Digout2Config5.
         * 
         * @param field data bitfield
         * @return data_source_b as a enum:DATA_SOURCE
         */
        public static int extractDigout2Config5_DataSourceB(long field) {
            return ((int) (field >> 44)) & 0xf;
        }

        /**
         * Extracts Enable the digout slot from Digout2Config6.
         * 
         * @param field data bitfield
         * @return slot_enabled as a bool
         */
        public static boolean extractDigout2Config6_SlotEnabled(long field) {
            return ((field >> 0) & 1) > 0;
        }

        /**
         * Extracts How the digout slot interacts with the next slot from Digout2Config6.
         * 
         * @param field data bitfield
         * @return next_slot_action as a enum:NEXT_SLOT_ACTION
         */
        public static int extractDigout2Config6_NextSlotAction(long field) {
            return ((int) (field >> 1)) & 0x3;
        }

        /**
         * Extracts Invert the digout slot's boolean value from Digout2Config6.
         * 
         * @param field data bitfield
         * @return invert_value as a bool
         */
        public static boolean extractDigout2Config6_InvertValue(long field) {
            return ((field >> 3) & 1) > 0;
        }

        /**
         * Extracts Opcode from Digout2Config6.
         * 
         * @param field data bitfield
         * @return opcode as a enum:SLOT_OPCODE
         */
        public static int extractDigout2Config6_Opcode(long field) {
            return ((int) (field >> 4)) & 0x7f;
        }

        /**
         * Extracts Additive immidiate from Digout2Config6.
         * 
         * @param field data bitfield
         * @return immidiate_additive as a sint:21
         */
        public static int extractDigout2Config6_ImmidiateAdditive(long field) {
            return ((((int) (field >> 11)) & 0x1fffff) << 11) >> 11;
        }

        /**
         * Extracts Scaling immidiate from Digout2Config6.
         * 
         * @param field data bitfield
         * @return immidiate_scaling as a uint:8
         */
        public static int extractDigout2Config6_ImmidiateScaling(long field) {
            return ((int) (field >> 32)) & 0xff;
        }

        /**
         * Extracts First ``a`` data source from Digout2Config6.
         * 
         * @param field data bitfield
         * @return data_source_a as a enum:DATA_SOURCE
         */
        public static int extractDigout2Config6_DataSourceA(long field) {
            return ((int) (field >> 40)) & 0xf;
        }

        /**
         * Extracts Second ``b`` data source from Digout2Config6.
         * 
         * @param field data bitfield
         * @return data_source_b as a enum:DATA_SOURCE
         */
        public static int extractDigout2Config6_DataSourceB(long field) {
            return ((int) (field >> 44)) & 0xf;
        }

        /**
         * Extracts Enable the digout slot from Digout2Config7.
         * 
         * @param field data bitfield
         * @return slot_enabled as a bool
         */
        public static boolean extractDigout2Config7_SlotEnabled(long field) {
            return ((field >> 0) & 1) > 0;
        }

        /**
         * Extracts How the digout slot interacts with the next slot from Digout2Config7.
         * 
         * @param field data bitfield
         * @return next_slot_action as a enum:NEXT_SLOT_ACTION
         */
        public static int extractDigout2Config7_NextSlotAction(long field) {
            return ((int) (field >> 1)) & 0x3;
        }

        /**
         * Extracts Invert the digout slot's boolean value from Digout2Config7.
         * 
         * @param field data bitfield
         * @return invert_value as a bool
         */
        public static boolean extractDigout2Config7_InvertValue(long field) {
            return ((field >> 3) & 1) > 0;
        }

        /**
         * Extracts Opcode from Digout2Config7.
         * 
         * @param field data bitfield
         * @return opcode as a enum:SLOT_OPCODE
         */
        public static int extractDigout2Config7_Opcode(long field) {
            return ((int) (field >> 4)) & 0x7f;
        }

        /**
         * Extracts Additive immidiate from Digout2Config7.
         * 
         * @param field data bitfield
         * @return immidiate_additive as a sint:21
         */
        public static int extractDigout2Config7_ImmidiateAdditive(long field) {
            return ((((int) (field >> 11)) & 0x1fffff) << 11) >> 11;
        }

        /**
         * Extracts Scaling immidiate from Digout2Config7.
         * 
         * @param field data bitfield
         * @return immidiate_scaling as a uint:8
         */
        public static int extractDigout2Config7_ImmidiateScaling(long field) {
            return ((int) (field >> 32)) & 0xff;
        }

        /**
         * Extracts First ``a`` data source from Digout2Config7.
         * 
         * @param field data bitfield
         * @return data_source_a as a enum:DATA_SOURCE
         */
        public static int extractDigout2Config7_DataSourceA(long field) {
            return ((int) (field >> 40)) & 0xf;
        }

        /**
         * Extracts Second ``b`` data source from Digout2Config7.
         * 
         * @param field data bitfield
         * @return data_source_b as a enum:DATA_SOURCE
         */
        public static int extractDigout2Config7_DataSourceB(long field) {
            return ((int) (field >> 44)) & 0xf;
        }

        /**
         * Extracts Enable the digout slot from Digout2Config8.
         * 
         * @param field data bitfield
         * @return slot_enabled as a bool
         */
        public static boolean extractDigout2Config8_SlotEnabled(long field) {
            return ((field >> 0) & 1) > 0;
        }

        /**
         * Extracts How the digout slot interacts with the next slot from Digout2Config8.
         * 
         * @param field data bitfield
         * @return next_slot_action as a enum:NEXT_SLOT_ACTION
         */
        public static int extractDigout2Config8_NextSlotAction(long field) {
            return ((int) (field >> 1)) & 0x3;
        }

        /**
         * Extracts Invert the digout slot's boolean value from Digout2Config8.
         * 
         * @param field data bitfield
         * @return invert_value as a bool
         */
        public static boolean extractDigout2Config8_InvertValue(long field) {
            return ((field >> 3) & 1) > 0;
        }

        /**
         * Extracts Opcode from Digout2Config8.
         * 
         * @param field data bitfield
         * @return opcode as a enum:SLOT_OPCODE
         */
        public static int extractDigout2Config8_Opcode(long field) {
            return ((int) (field >> 4)) & 0x7f;
        }

        /**
         * Extracts Additive immidiate from Digout2Config8.
         * 
         * @param field data bitfield
         * @return immidiate_additive as a sint:21
         */
        public static int extractDigout2Config8_ImmidiateAdditive(long field) {
            return ((((int) (field >> 11)) & 0x1fffff) << 11) >> 11;
        }

        /**
         * Extracts Scaling immidiate from Digout2Config8.
         * 
         * @param field data bitfield
         * @return immidiate_scaling as a uint:8
         */
        public static int extractDigout2Config8_ImmidiateScaling(long field) {
            return ((int) (field >> 32)) & 0xff;
        }

        /**
         * Extracts First ``a`` data source from Digout2Config8.
         * 
         * @param field data bitfield
         * @return data_source_a as a enum:DATA_SOURCE
         */
        public static int extractDigout2Config8_DataSourceA(long field) {
            return ((int) (field >> 40)) & 0xf;
        }

        /**
         * Extracts Second ``b`` data source from Digout2Config8.
         * 
         * @param field data bitfield
         * @return data_source_b as a enum:DATA_SOURCE
         */
        public static int extractDigout2Config8_DataSourceB(long field) {
            return ((int) (field >> 44)) & 0xf;
        }

        /**
         * Extracts Enable the digout slot from Digout2Config9.
         * 
         * @param field data bitfield
         * @return slot_enabled as a bool
         */
        public static boolean extractDigout2Config9_SlotEnabled(long field) {
            return ((field >> 0) & 1) > 0;
        }

        /**
         * Extracts How the digout slot interacts with the next slot from Digout2Config9.
         * 
         * @param field data bitfield
         * @return next_slot_action as a enum:NEXT_SLOT_ACTION
         */
        public static int extractDigout2Config9_NextSlotAction(long field) {
            return ((int) (field >> 1)) & 0x3;
        }

        /**
         * Extracts Invert the digout slot's boolean value from Digout2Config9.
         * 
         * @param field data bitfield
         * @return invert_value as a bool
         */
        public static boolean extractDigout2Config9_InvertValue(long field) {
            return ((field >> 3) & 1) > 0;
        }

        /**
         * Extracts Opcode from Digout2Config9.
         * 
         * @param field data bitfield
         * @return opcode as a enum:SLOT_OPCODE
         */
        public static int extractDigout2Config9_Opcode(long field) {
            return ((int) (field >> 4)) & 0x7f;
        }

        /**
         * Extracts Additive immidiate from Digout2Config9.
         * 
         * @param field data bitfield
         * @return immidiate_additive as a sint:21
         */
        public static int extractDigout2Config9_ImmidiateAdditive(long field) {
            return ((((int) (field >> 11)) & 0x1fffff) << 11) >> 11;
        }

        /**
         * Extracts Scaling immidiate from Digout2Config9.
         * 
         * @param field data bitfield
         * @return immidiate_scaling as a uint:8
         */
        public static int extractDigout2Config9_ImmidiateScaling(long field) {
            return ((int) (field >> 32)) & 0xff;
        }

        /**
         * Extracts First ``a`` data source from Digout2Config9.
         * 
         * @param field data bitfield
         * @return data_source_a as a enum:DATA_SOURCE
         */
        public static int extractDigout2Config9_DataSourceA(long field) {
            return ((int) (field >> 40)) & 0xf;
        }

        /**
         * Extracts Second ``b`` data source from Digout2Config9.
         * 
         * @param field data bitfield
         * @return data_source_b as a enum:DATA_SOURCE
         */
        public static int extractDigout2Config9_DataSourceB(long field) {
            return ((int) (field >> 44)) & 0xf;
        }

        /**
         * Extracts Enable the digout slot from Digout2Config10.
         * 
         * @param field data bitfield
         * @return slot_enabled as a bool
         */
        public static boolean extractDigout2Config10_SlotEnabled(long field) {
            return ((field >> 0) & 1) > 0;
        }

        /**
         * Extracts How the digout slot interacts with the next slot from Digout2Config10.
         * 
         * @param field data bitfield
         * @return next_slot_action as a enum:NEXT_SLOT_ACTION
         */
        public static int extractDigout2Config10_NextSlotAction(long field) {
            return ((int) (field >> 1)) & 0x3;
        }

        /**
         * Extracts Invert the digout slot's boolean value from Digout2Config10.
         * 
         * @param field data bitfield
         * @return invert_value as a bool
         */
        public static boolean extractDigout2Config10_InvertValue(long field) {
            return ((field >> 3) & 1) > 0;
        }

        /**
         * Extracts Opcode from Digout2Config10.
         * 
         * @param field data bitfield
         * @return opcode as a enum:SLOT_OPCODE
         */
        public static int extractDigout2Config10_Opcode(long field) {
            return ((int) (field >> 4)) & 0x7f;
        }

        /**
         * Extracts Additive immidiate from Digout2Config10.
         * 
         * @param field data bitfield
         * @return immidiate_additive as a sint:21
         */
        public static int extractDigout2Config10_ImmidiateAdditive(long field) {
            return ((((int) (field >> 11)) & 0x1fffff) << 11) >> 11;
        }

        /**
         * Extracts Scaling immidiate from Digout2Config10.
         * 
         * @param field data bitfield
         * @return immidiate_scaling as a uint:8
         */
        public static int extractDigout2Config10_ImmidiateScaling(long field) {
            return ((int) (field >> 32)) & 0xff;
        }

        /**
         * Extracts First ``a`` data source from Digout2Config10.
         * 
         * @param field data bitfield
         * @return data_source_a as a enum:DATA_SOURCE
         */
        public static int extractDigout2Config10_DataSourceA(long field) {
            return ((int) (field >> 40)) & 0xf;
        }

        /**
         * Extracts Second ``b`` data source from Digout2Config10.
         * 
         * @param field data bitfield
         * @return data_source_b as a enum:DATA_SOURCE
         */
        public static int extractDigout2Config10_DataSourceB(long field) {
            return ((int) (field >> 44)) & 0xf;
        }

        /**
         * Extracts Enable the digout slot from Digout2Config11.
         * 
         * @param field data bitfield
         * @return slot_enabled as a bool
         */
        public static boolean extractDigout2Config11_SlotEnabled(long field) {
            return ((field >> 0) & 1) > 0;
        }

        /**
         * Extracts How the digout slot interacts with the next slot from Digout2Config11.
         * 
         * @param field data bitfield
         * @return next_slot_action as a enum:NEXT_SLOT_ACTION
         */
        public static int extractDigout2Config11_NextSlotAction(long field) {
            return ((int) (field >> 1)) & 0x3;
        }

        /**
         * Extracts Invert the digout slot's boolean value from Digout2Config11.
         * 
         * @param field data bitfield
         * @return invert_value as a bool
         */
        public static boolean extractDigout2Config11_InvertValue(long field) {
            return ((field >> 3) & 1) > 0;
        }

        /**
         * Extracts Opcode from Digout2Config11.
         * 
         * @param field data bitfield
         * @return opcode as a enum:SLOT_OPCODE
         */
        public static int extractDigout2Config11_Opcode(long field) {
            return ((int) (field >> 4)) & 0x7f;
        }

        /**
         * Extracts Additive immidiate from Digout2Config11.
         * 
         * @param field data bitfield
         * @return immidiate_additive as a sint:21
         */
        public static int extractDigout2Config11_ImmidiateAdditive(long field) {
            return ((((int) (field >> 11)) & 0x1fffff) << 11) >> 11;
        }

        /**
         * Extracts Scaling immidiate from Digout2Config11.
         * 
         * @param field data bitfield
         * @return immidiate_scaling as a uint:8
         */
        public static int extractDigout2Config11_ImmidiateScaling(long field) {
            return ((int) (field >> 32)) & 0xff;
        }

        /**
         * Extracts First ``a`` data source from Digout2Config11.
         * 
         * @param field data bitfield
         * @return data_source_a as a enum:DATA_SOURCE
         */
        public static int extractDigout2Config11_DataSourceA(long field) {
            return ((int) (field >> 40)) & 0xf;
        }

        /**
         * Extracts Second ``b`` data source from Digout2Config11.
         * 
         * @param field data bitfield
         * @return data_source_b as a enum:DATA_SOURCE
         */
        public static int extractDigout2Config11_DataSourceB(long field) {
            return ((int) (field >> 44)) & 0xf;
        }

        /**
         * Extracts Enable the digout slot from Digout2Config12.
         * 
         * @param field data bitfield
         * @return slot_enabled as a bool
         */
        public static boolean extractDigout2Config12_SlotEnabled(long field) {
            return ((field >> 0) & 1) > 0;
        }

        /**
         * Extracts How the digout slot interacts with the next slot from Digout2Config12.
         * 
         * @param field data bitfield
         * @return next_slot_action as a enum:NEXT_SLOT_ACTION
         */
        public static int extractDigout2Config12_NextSlotAction(long field) {
            return ((int) (field >> 1)) & 0x3;
        }

        /**
         * Extracts Invert the digout slot's boolean value from Digout2Config12.
         * 
         * @param field data bitfield
         * @return invert_value as a bool
         */
        public static boolean extractDigout2Config12_InvertValue(long field) {
            return ((field >> 3) & 1) > 0;
        }

        /**
         * Extracts Opcode from Digout2Config12.
         * 
         * @param field data bitfield
         * @return opcode as a enum:SLOT_OPCODE
         */
        public static int extractDigout2Config12_Opcode(long field) {
            return ((int) (field >> 4)) & 0x7f;
        }

        /**
         * Extracts Additive immidiate from Digout2Config12.
         * 
         * @param field data bitfield
         * @return immidiate_additive as a sint:21
         */
        public static int extractDigout2Config12_ImmidiateAdditive(long field) {
            return ((((int) (field >> 11)) & 0x1fffff) << 11) >> 11;
        }

        /**
         * Extracts Scaling immidiate from Digout2Config12.
         * 
         * @param field data bitfield
         * @return immidiate_scaling as a uint:8
         */
        public static int extractDigout2Config12_ImmidiateScaling(long field) {
            return ((int) (field >> 32)) & 0xff;
        }

        /**
         * Extracts First ``a`` data source from Digout2Config12.
         * 
         * @param field data bitfield
         * @return data_source_a as a enum:DATA_SOURCE
         */
        public static int extractDigout2Config12_DataSourceA(long field) {
            return ((int) (field >> 40)) & 0xf;
        }

        /**
         * Extracts Second ``b`` data source from Digout2Config12.
         * 
         * @param field data bitfield
         * @return data_source_b as a enum:DATA_SOURCE
         */
        public static int extractDigout2Config12_DataSourceB(long field) {
            return ((int) (field >> 44)) & 0xf;
        }

        /**
         * Extracts Enable the digout slot from Digout2Config13.
         * 
         * @param field data bitfield
         * @return slot_enabled as a bool
         */
        public static boolean extractDigout2Config13_SlotEnabled(long field) {
            return ((field >> 0) & 1) > 0;
        }

        /**
         * Extracts How the digout slot interacts with the next slot from Digout2Config13.
         * 
         * @param field data bitfield
         * @return next_slot_action as a enum:NEXT_SLOT_ACTION
         */
        public static int extractDigout2Config13_NextSlotAction(long field) {
            return ((int) (field >> 1)) & 0x3;
        }

        /**
         * Extracts Invert the digout slot's boolean value from Digout2Config13.
         * 
         * @param field data bitfield
         * @return invert_value as a bool
         */
        public static boolean extractDigout2Config13_InvertValue(long field) {
            return ((field >> 3) & 1) > 0;
        }

        /**
         * Extracts Opcode from Digout2Config13.
         * 
         * @param field data bitfield
         * @return opcode as a enum:SLOT_OPCODE
         */
        public static int extractDigout2Config13_Opcode(long field) {
            return ((int) (field >> 4)) & 0x7f;
        }

        /**
         * Extracts Additive immidiate from Digout2Config13.
         * 
         * @param field data bitfield
         * @return immidiate_additive as a sint:21
         */
        public static int extractDigout2Config13_ImmidiateAdditive(long field) {
            return ((((int) (field >> 11)) & 0x1fffff) << 11) >> 11;
        }

        /**
         * Extracts Scaling immidiate from Digout2Config13.
         * 
         * @param field data bitfield
         * @return immidiate_scaling as a uint:8
         */
        public static int extractDigout2Config13_ImmidiateScaling(long field) {
            return ((int) (field >> 32)) & 0xff;
        }

        /**
         * Extracts First ``a`` data source from Digout2Config13.
         * 
         * @param field data bitfield
         * @return data_source_a as a enum:DATA_SOURCE
         */
        public static int extractDigout2Config13_DataSourceA(long field) {
            return ((int) (field >> 40)) & 0xf;
        }

        /**
         * Extracts Second ``b`` data source from Digout2Config13.
         * 
         * @param field data bitfield
         * @return data_source_b as a enum:DATA_SOURCE
         */
        public static int extractDigout2Config13_DataSourceB(long field) {
            return ((int) (field >> 44)) & 0xf;
        }

        /**
         * Extracts Enable the digout slot from Digout2Config14.
         * 
         * @param field data bitfield
         * @return slot_enabled as a bool
         */
        public static boolean extractDigout2Config14_SlotEnabled(long field) {
            return ((field >> 0) & 1) > 0;
        }

        /**
         * Extracts How the digout slot interacts with the next slot from Digout2Config14.
         * 
         * @param field data bitfield
         * @return next_slot_action as a enum:NEXT_SLOT_ACTION
         */
        public static int extractDigout2Config14_NextSlotAction(long field) {
            return ((int) (field >> 1)) & 0x3;
        }

        /**
         * Extracts Invert the digout slot's boolean value from Digout2Config14.
         * 
         * @param field data bitfield
         * @return invert_value as a bool
         */
        public static boolean extractDigout2Config14_InvertValue(long field) {
            return ((field >> 3) & 1) > 0;
        }

        /**
         * Extracts Opcode from Digout2Config14.
         * 
         * @param field data bitfield
         * @return opcode as a enum:SLOT_OPCODE
         */
        public static int extractDigout2Config14_Opcode(long field) {
            return ((int) (field >> 4)) & 0x7f;
        }

        /**
         * Extracts Additive immidiate from Digout2Config14.
         * 
         * @param field data bitfield
         * @return immidiate_additive as a sint:21
         */
        public static int extractDigout2Config14_ImmidiateAdditive(long field) {
            return ((((int) (field >> 11)) & 0x1fffff) << 11) >> 11;
        }

        /**
         * Extracts Scaling immidiate from Digout2Config14.
         * 
         * @param field data bitfield
         * @return immidiate_scaling as a uint:8
         */
        public static int extractDigout2Config14_ImmidiateScaling(long field) {
            return ((int) (field >> 32)) & 0xff;
        }

        /**
         * Extracts First ``a`` data source from Digout2Config14.
         * 
         * @param field data bitfield
         * @return data_source_a as a enum:DATA_SOURCE
         */
        public static int extractDigout2Config14_DataSourceA(long field) {
            return ((int) (field >> 40)) & 0xf;
        }

        /**
         * Extracts Second ``b`` data source from Digout2Config14.
         * 
         * @param field data bitfield
         * @return data_source_b as a enum:DATA_SOURCE
         */
        public static int extractDigout2Config14_DataSourceB(long field) {
            return ((int) (field >> 44)) & 0xf;
        }

        /**
         * Extracts Enable the digout slot from Digout2Config15.
         * 
         * @param field data bitfield
         * @return slot_enabled as a bool
         */
        public static boolean extractDigout2Config15_SlotEnabled(long field) {
            return ((field >> 0) & 1) > 0;
        }

        /**
         * Extracts How the digout slot interacts with the next slot from Digout2Config15.
         * 
         * @param field data bitfield
         * @return next_slot_action as a enum:NEXT_SLOT_ACTION
         */
        public static int extractDigout2Config15_NextSlotAction(long field) {
            return ((int) (field >> 1)) & 0x3;
        }

        /**
         * Extracts Invert the digout slot's boolean value from Digout2Config15.
         * 
         * @param field data bitfield
         * @return invert_value as a bool
         */
        public static boolean extractDigout2Config15_InvertValue(long field) {
            return ((field >> 3) & 1) > 0;
        }

        /**
         * Extracts Opcode from Digout2Config15.
         * 
         * @param field data bitfield
         * @return opcode as a enum:SLOT_OPCODE
         */
        public static int extractDigout2Config15_Opcode(long field) {
            return ((int) (field >> 4)) & 0x7f;
        }

        /**
         * Extracts Additive immidiate from Digout2Config15.
         * 
         * @param field data bitfield
         * @return immidiate_additive as a sint:21
         */
        public static int extractDigout2Config15_ImmidiateAdditive(long field) {
            return ((((int) (field >> 11)) & 0x1fffff) << 11) >> 11;
        }

        /**
         * Extracts Scaling immidiate from Digout2Config15.
         * 
         * @param field data bitfield
         * @return immidiate_scaling as a uint:8
         */
        public static int extractDigout2Config15_ImmidiateScaling(long field) {
            return ((int) (field >> 32)) & 0xff;
        }

        /**
         * Extracts First ``a`` data source from Digout2Config15.
         * 
         * @param field data bitfield
         * @return data_source_a as a enum:DATA_SOURCE
         */
        public static int extractDigout2Config15_DataSourceA(long field) {
            return ((int) (field >> 40)) & 0xf;
        }

        /**
         * Extracts Second ``b`` data source from Digout2Config15.
         * 
         * @param field data bitfield
         * @return data_source_b as a enum:DATA_SOURCE
         */
        public static int extractDigout2Config15_DataSourceB(long field) {
            return ((int) (field >> 44)) & 0xf;
        }

        /**
         * Extracts Device-specific type identifier from DeviceType.
         * 
         * @param field data bitfield
         * @return DEVICE_TYPE as a uint:16
         */
        public static int extractDeviceType(long field) {
            return ((int) field) & 0xffff;
        }

        /**
         * Extracts Firmware version patch number from FirmwareVersion.
         * 
         * @param field data bitfield
         * @return firmware_patch as a uint:8
         */
        public static int extractFirmwareVersion_FirmwarePatch(long field) {
            return ((int) field) & 0xff;
        }

        /**
         * Extracts Firmware version minor number from FirmwareVersion.
         * 
         * @param field data bitfield
         * @return firmware_minor as a uint:8
         */
        public static int extractFirmwareVersion_FirmwareMinor(long field) {
            return ((int) (field >> 8)) & 0xff;
        }

        /**
         * Extracts Firmware version year from FirmwareVersion.
         * 
         * @param field data bitfield
         * @return firmware_year as a uint:16
         */
        public static int extractFirmwareVersion_FirmwareYear(long field) {
            return ((int) (field >> 16)) & 0xffff;
        }

        /**
         * Extracts Serial number from SerialNumber.
         * 
         * @param field data bitfield
         * @return SERIAL_NUMBER as a buf:48
         */
        public static long extractSerialNumber(long field) {
            return field & 0xffffffffffffL;
        }

        /**
         * Extracts Status frame period (ms) from StatusFramePeriod.
         * 
         * @param field data bitfield
         * @return STATUS_FRAME_PERIOD as a uint:16
         */
        public static int extractStatusFramePeriod(long field) {
            return ((int) field) & 0xffff;
        }

        /**
         * Constructs a DISTANCE_FRAME_PERIOD setting.
         * 
         * @param distanceFramePeriod Distance frame period (ms) (uint:16)
         * @return message data as long
         */
        public static long constructDistanceFramePeriod(int distanceFramePeriod) {
            if (distanceFramePeriod < 0 || distanceFramePeriod > 65535) { throw new IllegalArgumentException("distanceFramePeriod must be between [0..=65535] inclusive, instead got " + distanceFramePeriod); }
            return ((long) distanceFramePeriod);
        }

        /**
         * Constructs a COLOR_FRAME_PERIOD setting.
         * 
         * @param colorFramePeriod Color frame period (ms) (uint:16)
         * @return message data as long
         */
        public static long constructColorFramePeriod(int colorFramePeriod) {
            if (colorFramePeriod < 0 || colorFramePeriod > 65535) { throw new IllegalArgumentException("colorFramePeriod must be between [0..=65535] inclusive, instead got " + colorFramePeriod); }
            return ((long) colorFramePeriod);
        }

        /**
         * Constructs a DIGOUT_FRAME_PERIOD setting.
         * 
         * @param digoutFramePeriod Digout frame period (ms) (uint:16)
         * @return message data as long
         */
        public static long constructDigoutFramePeriod(int digoutFramePeriod) {
            if (digoutFramePeriod < 0 || digoutFramePeriod > 65535) { throw new IllegalArgumentException("digoutFramePeriod must be between [0..=65535] inclusive, instead got " + digoutFramePeriod); }
            return ((long) digoutFramePeriod);
        }

        /**
         * Constructs a DISTANCE_EXTRA_FRAME_MODE setting.
         * 
         * @param distanceExtraFrameMode Distance extra frame mode (enum:EXTRA_FRAME_MODE)
         * @return message data as long
         */
        public static long constructDistanceExtraFrameMode(int distanceExtraFrameMode) {
            return ((long) distanceExtraFrameMode);
        }

        /**
         * Constructs a COLOR_EXTRA_FRAME_MODE setting.
         * 
         * @param colorExtraFrameMode Color extra frame frame mode (enum:EXTRA_FRAME_MODE)
         * @return message data as long
         */
        public static long constructColorExtraFrameMode(int colorExtraFrameMode) {
            return ((long) colorExtraFrameMode);
        }

        /**
         * Constructs a LAMP_BRIGHTNESS setting.
         * 
         * @param lampBrightness Lamp LED brightness (uint:16)
         * @return message data as long
         */
        public static long constructLampBrightness(int lampBrightness) {
            if (lampBrightness < 0 || lampBrightness > 36000) { throw new IllegalArgumentException("lampBrightness must be between [0..=36000] inclusive, instead got " + lampBrightness); }
            return ((long) lampBrightness);
        }

        /**
         * Constructs a COLOR_INTEGRATION_PERIOD setting.
         * 
         * @param colorIntegrationPeriod Color integration period (enum:COLOR_INTEGRATION_PERIOD)
         * @return message data as long
         */
        public static long constructColorIntegrationPeriod(int colorIntegrationPeriod) {
            return ((long) colorIntegrationPeriod);
        }

        /**
         * Constructs a DISTANCE_INTEGRATION_PERIOD setting.
         * 
         * @param distanceIntegrationPeriod Distance integration period (enum:DISTANCE_INTEGRATION_PERIOD)
         * @return message data as long
         */
        public static long constructDistanceIntegrationPeriod(int distanceIntegrationPeriod) {
            return ((long) distanceIntegrationPeriod);
        }

        /**
         * Constructs a DIGOUT1_OUTPUT_CONFIG setting.
         * 
         * @param outputConfig Enable digout pad (enum:DIGOUT_OUTPUT_CONFIG)
         * @param pwmDataSource The data source to use in PWM mode. (enum:DATA_SOURCE)
         * @return message data as long
         */
        public static long constructDigout1OutputConfig(int outputConfig, int pwmDataSource) {
            return ((long) outputConfig) | 
                (((long) pwmDataSource) << 8);
        }

        /**
         * Constructs a DIGOUT2_OUTPUT_CONFIG setting.
         * 
         * @param outputConfig Enable digout pad (enum:DIGOUT_OUTPUT_CONFIG)
         * @param pwmDataSource The data source to use in PWM mode. (enum:DATA_SOURCE)
         * @return message data as long
         */
        public static long constructDigout2OutputConfig(int outputConfig, int pwmDataSource) {
            return ((long) outputConfig) | 
                (((long) pwmDataSource) << 8);
        }

        /**
         * Constructs a DIGOUT1_MESSAGE_ON_CHANGE setting.
         * 
         * @param positiveEdge Send digout message on positive edge (false-&gt;true) (bool)
         * @param negativeEdge Send digout message on negative edge (true-&gt;false) (bool)
         * @return message data as long
         */
        public static long constructDigout1MessageOnChange(boolean positiveEdge, boolean negativeEdge) {
            return (positiveEdge ? 1L : 0L) | 
                ((negativeEdge ? 1L : 0L) << 1);
        }

        /**
         * Constructs a DIGOUT2_MESSAGE_ON_CHANGE setting.
         * 
         * @param positiveEdge Send digout message on positive edge (false-&gt;true) (bool)
         * @param negativeEdge Send digout message on negative edge (true-&gt;false) (bool)
         * @return message data as long
         */
        public static long constructDigout2MessageOnChange(boolean positiveEdge, boolean negativeEdge) {
            return (positiveEdge ? 1L : 0L) | 
                ((negativeEdge ? 1L : 0L) << 1);
        }

        /**
         * Constructs a DIGOUT1_CONFIG_0 setting.
         * 
         * @param slotEnabled Enable the digout slot (bool)
         * @param nextSlotAction How the digout slot interacts with the next slot (enum:NEXT_SLOT_ACTION)
         * @param invertValue Invert the digout slot's boolean value (bool)
         * @param opcode Opcode (enum:SLOT_OPCODE)
         * @param immidiateAdditive Additive immidiate (sint:21)
         * @param immidiateScaling Scaling immidiate (uint:8)
         * @param dataSourceA First ``a`` data source (enum:DATA_SOURCE)
         * @param dataSourceB Second ``b`` data source (enum:DATA_SOURCE)
         * @return message data as long
         */
        public static long constructDigout1Config0(boolean slotEnabled, int nextSlotAction, boolean invertValue, int opcode, int immidiateAdditive, int immidiateScaling, int dataSourceA, int dataSourceB) {
            if (immidiateAdditive < -1048576 || immidiateAdditive > 1048575) { throw new IllegalArgumentException("immidiateAdditive must be between [-1048576..=1048575] inclusive, instead got " + immidiateAdditive); }
            if (immidiateScaling < 0 || immidiateScaling > 255) { throw new IllegalArgumentException("immidiateScaling must be between [0..=255] inclusive, instead got " + immidiateScaling); }
            return (slotEnabled ? 1L : 0L) | 
                (((long) nextSlotAction) << 1) | 
                ((invertValue ? 1L : 0L) << 3) | 
                (((long) opcode) << 4) | 
                (((long) immidiateAdditive) << 11) | 
                (((long) immidiateScaling) << 32) | 
                (((long) dataSourceA) << 40) | 
                (((long) dataSourceB) << 44);
        }

        /**
         * Constructs a DIGOUT1_CONFIG_1 setting.
         * 
         * @param slotEnabled Enable the digout slot (bool)
         * @param nextSlotAction How the digout slot interacts with the next slot (enum:NEXT_SLOT_ACTION)
         * @param invertValue Invert the digout slot's boolean value (bool)
         * @param opcode Opcode (enum:SLOT_OPCODE)
         * @param immidiateAdditive Additive immidiate (sint:21)
         * @param immidiateScaling Scaling immidiate (uint:8)
         * @param dataSourceA First ``a`` data source (enum:DATA_SOURCE)
         * @param dataSourceB Second ``b`` data source (enum:DATA_SOURCE)
         * @return message data as long
         */
        public static long constructDigout1Config1(boolean slotEnabled, int nextSlotAction, boolean invertValue, int opcode, int immidiateAdditive, int immidiateScaling, int dataSourceA, int dataSourceB) {
            if (immidiateAdditive < -1048576 || immidiateAdditive > 1048575) { throw new IllegalArgumentException("immidiateAdditive must be between [-1048576..=1048575] inclusive, instead got " + immidiateAdditive); }
            if (immidiateScaling < 0 || immidiateScaling > 255) { throw new IllegalArgumentException("immidiateScaling must be between [0..=255] inclusive, instead got " + immidiateScaling); }
            return (slotEnabled ? 1L : 0L) | 
                (((long) nextSlotAction) << 1) | 
                ((invertValue ? 1L : 0L) << 3) | 
                (((long) opcode) << 4) | 
                (((long) immidiateAdditive) << 11) | 
                (((long) immidiateScaling) << 32) | 
                (((long) dataSourceA) << 40) | 
                (((long) dataSourceB) << 44);
        }

        /**
         * Constructs a DIGOUT1_CONFIG_2 setting.
         * 
         * @param slotEnabled Enable the digout slot (bool)
         * @param nextSlotAction How the digout slot interacts with the next slot (enum:NEXT_SLOT_ACTION)
         * @param invertValue Invert the digout slot's boolean value (bool)
         * @param opcode Opcode (enum:SLOT_OPCODE)
         * @param immidiateAdditive Additive immidiate (sint:21)
         * @param immidiateScaling Scaling immidiate (uint:8)
         * @param dataSourceA First ``a`` data source (enum:DATA_SOURCE)
         * @param dataSourceB Second ``b`` data source (enum:DATA_SOURCE)
         * @return message data as long
         */
        public static long constructDigout1Config2(boolean slotEnabled, int nextSlotAction, boolean invertValue, int opcode, int immidiateAdditive, int immidiateScaling, int dataSourceA, int dataSourceB) {
            if (immidiateAdditive < -1048576 || immidiateAdditive > 1048575) { throw new IllegalArgumentException("immidiateAdditive must be between [-1048576..=1048575] inclusive, instead got " + immidiateAdditive); }
            if (immidiateScaling < 0 || immidiateScaling > 255) { throw new IllegalArgumentException("immidiateScaling must be between [0..=255] inclusive, instead got " + immidiateScaling); }
            return (slotEnabled ? 1L : 0L) | 
                (((long) nextSlotAction) << 1) | 
                ((invertValue ? 1L : 0L) << 3) | 
                (((long) opcode) << 4) | 
                (((long) immidiateAdditive) << 11) | 
                (((long) immidiateScaling) << 32) | 
                (((long) dataSourceA) << 40) | 
                (((long) dataSourceB) << 44);
        }

        /**
         * Constructs a DIGOUT1_CONFIG_3 setting.
         * 
         * @param slotEnabled Enable the digout slot (bool)
         * @param nextSlotAction How the digout slot interacts with the next slot (enum:NEXT_SLOT_ACTION)
         * @param invertValue Invert the digout slot's boolean value (bool)
         * @param opcode Opcode (enum:SLOT_OPCODE)
         * @param immidiateAdditive Additive immidiate (sint:21)
         * @param immidiateScaling Scaling immidiate (uint:8)
         * @param dataSourceA First ``a`` data source (enum:DATA_SOURCE)
         * @param dataSourceB Second ``b`` data source (enum:DATA_SOURCE)
         * @return message data as long
         */
        public static long constructDigout1Config3(boolean slotEnabled, int nextSlotAction, boolean invertValue, int opcode, int immidiateAdditive, int immidiateScaling, int dataSourceA, int dataSourceB) {
            if (immidiateAdditive < -1048576 || immidiateAdditive > 1048575) { throw new IllegalArgumentException("immidiateAdditive must be between [-1048576..=1048575] inclusive, instead got " + immidiateAdditive); }
            if (immidiateScaling < 0 || immidiateScaling > 255) { throw new IllegalArgumentException("immidiateScaling must be between [0..=255] inclusive, instead got " + immidiateScaling); }
            return (slotEnabled ? 1L : 0L) | 
                (((long) nextSlotAction) << 1) | 
                ((invertValue ? 1L : 0L) << 3) | 
                (((long) opcode) << 4) | 
                (((long) immidiateAdditive) << 11) | 
                (((long) immidiateScaling) << 32) | 
                (((long) dataSourceA) << 40) | 
                (((long) dataSourceB) << 44);
        }

        /**
         * Constructs a DIGOUT1_CONFIG_4 setting.
         * 
         * @param slotEnabled Enable the digout slot (bool)
         * @param nextSlotAction How the digout slot interacts with the next slot (enum:NEXT_SLOT_ACTION)
         * @param invertValue Invert the digout slot's boolean value (bool)
         * @param opcode Opcode (enum:SLOT_OPCODE)
         * @param immidiateAdditive Additive immidiate (sint:21)
         * @param immidiateScaling Scaling immidiate (uint:8)
         * @param dataSourceA First ``a`` data source (enum:DATA_SOURCE)
         * @param dataSourceB Second ``b`` data source (enum:DATA_SOURCE)
         * @return message data as long
         */
        public static long constructDigout1Config4(boolean slotEnabled, int nextSlotAction, boolean invertValue, int opcode, int immidiateAdditive, int immidiateScaling, int dataSourceA, int dataSourceB) {
            if (immidiateAdditive < -1048576 || immidiateAdditive > 1048575) { throw new IllegalArgumentException("immidiateAdditive must be between [-1048576..=1048575] inclusive, instead got " + immidiateAdditive); }
            if (immidiateScaling < 0 || immidiateScaling > 255) { throw new IllegalArgumentException("immidiateScaling must be between [0..=255] inclusive, instead got " + immidiateScaling); }
            return (slotEnabled ? 1L : 0L) | 
                (((long) nextSlotAction) << 1) | 
                ((invertValue ? 1L : 0L) << 3) | 
                (((long) opcode) << 4) | 
                (((long) immidiateAdditive) << 11) | 
                (((long) immidiateScaling) << 32) | 
                (((long) dataSourceA) << 40) | 
                (((long) dataSourceB) << 44);
        }

        /**
         * Constructs a DIGOUT1_CONFIG_5 setting.
         * 
         * @param slotEnabled Enable the digout slot (bool)
         * @param nextSlotAction How the digout slot interacts with the next slot (enum:NEXT_SLOT_ACTION)
         * @param invertValue Invert the digout slot's boolean value (bool)
         * @param opcode Opcode (enum:SLOT_OPCODE)
         * @param immidiateAdditive Additive immidiate (sint:21)
         * @param immidiateScaling Scaling immidiate (uint:8)
         * @param dataSourceA First ``a`` data source (enum:DATA_SOURCE)
         * @param dataSourceB Second ``b`` data source (enum:DATA_SOURCE)
         * @return message data as long
         */
        public static long constructDigout1Config5(boolean slotEnabled, int nextSlotAction, boolean invertValue, int opcode, int immidiateAdditive, int immidiateScaling, int dataSourceA, int dataSourceB) {
            if (immidiateAdditive < -1048576 || immidiateAdditive > 1048575) { throw new IllegalArgumentException("immidiateAdditive must be between [-1048576..=1048575] inclusive, instead got " + immidiateAdditive); }
            if (immidiateScaling < 0 || immidiateScaling > 255) { throw new IllegalArgumentException("immidiateScaling must be between [0..=255] inclusive, instead got " + immidiateScaling); }
            return (slotEnabled ? 1L : 0L) | 
                (((long) nextSlotAction) << 1) | 
                ((invertValue ? 1L : 0L) << 3) | 
                (((long) opcode) << 4) | 
                (((long) immidiateAdditive) << 11) | 
                (((long) immidiateScaling) << 32) | 
                (((long) dataSourceA) << 40) | 
                (((long) dataSourceB) << 44);
        }

        /**
         * Constructs a DIGOUT1_CONFIG_6 setting.
         * 
         * @param slotEnabled Enable the digout slot (bool)
         * @param nextSlotAction How the digout slot interacts with the next slot (enum:NEXT_SLOT_ACTION)
         * @param invertValue Invert the digout slot's boolean value (bool)
         * @param opcode Opcode (enum:SLOT_OPCODE)
         * @param immidiateAdditive Additive immidiate (sint:21)
         * @param immidiateScaling Scaling immidiate (uint:8)
         * @param dataSourceA First ``a`` data source (enum:DATA_SOURCE)
         * @param dataSourceB Second ``b`` data source (enum:DATA_SOURCE)
         * @return message data as long
         */
        public static long constructDigout1Config6(boolean slotEnabled, int nextSlotAction, boolean invertValue, int opcode, int immidiateAdditive, int immidiateScaling, int dataSourceA, int dataSourceB) {
            if (immidiateAdditive < -1048576 || immidiateAdditive > 1048575) { throw new IllegalArgumentException("immidiateAdditive must be between [-1048576..=1048575] inclusive, instead got " + immidiateAdditive); }
            if (immidiateScaling < 0 || immidiateScaling > 255) { throw new IllegalArgumentException("immidiateScaling must be between [0..=255] inclusive, instead got " + immidiateScaling); }
            return (slotEnabled ? 1L : 0L) | 
                (((long) nextSlotAction) << 1) | 
                ((invertValue ? 1L : 0L) << 3) | 
                (((long) opcode) << 4) | 
                (((long) immidiateAdditive) << 11) | 
                (((long) immidiateScaling) << 32) | 
                (((long) dataSourceA) << 40) | 
                (((long) dataSourceB) << 44);
        }

        /**
         * Constructs a DIGOUT1_CONFIG_7 setting.
         * 
         * @param slotEnabled Enable the digout slot (bool)
         * @param nextSlotAction How the digout slot interacts with the next slot (enum:NEXT_SLOT_ACTION)
         * @param invertValue Invert the digout slot's boolean value (bool)
         * @param opcode Opcode (enum:SLOT_OPCODE)
         * @param immidiateAdditive Additive immidiate (sint:21)
         * @param immidiateScaling Scaling immidiate (uint:8)
         * @param dataSourceA First ``a`` data source (enum:DATA_SOURCE)
         * @param dataSourceB Second ``b`` data source (enum:DATA_SOURCE)
         * @return message data as long
         */
        public static long constructDigout1Config7(boolean slotEnabled, int nextSlotAction, boolean invertValue, int opcode, int immidiateAdditive, int immidiateScaling, int dataSourceA, int dataSourceB) {
            if (immidiateAdditive < -1048576 || immidiateAdditive > 1048575) { throw new IllegalArgumentException("immidiateAdditive must be between [-1048576..=1048575] inclusive, instead got " + immidiateAdditive); }
            if (immidiateScaling < 0 || immidiateScaling > 255) { throw new IllegalArgumentException("immidiateScaling must be between [0..=255] inclusive, instead got " + immidiateScaling); }
            return (slotEnabled ? 1L : 0L) | 
                (((long) nextSlotAction) << 1) | 
                ((invertValue ? 1L : 0L) << 3) | 
                (((long) opcode) << 4) | 
                (((long) immidiateAdditive) << 11) | 
                (((long) immidiateScaling) << 32) | 
                (((long) dataSourceA) << 40) | 
                (((long) dataSourceB) << 44);
        }

        /**
         * Constructs a DIGOUT1_CONFIG_8 setting.
         * 
         * @param slotEnabled Enable the digout slot (bool)
         * @param nextSlotAction How the digout slot interacts with the next slot (enum:NEXT_SLOT_ACTION)
         * @param invertValue Invert the digout slot's boolean value (bool)
         * @param opcode Opcode (enum:SLOT_OPCODE)
         * @param immidiateAdditive Additive immidiate (sint:21)
         * @param immidiateScaling Scaling immidiate (uint:8)
         * @param dataSourceA First ``a`` data source (enum:DATA_SOURCE)
         * @param dataSourceB Second ``b`` data source (enum:DATA_SOURCE)
         * @return message data as long
         */
        public static long constructDigout1Config8(boolean slotEnabled, int nextSlotAction, boolean invertValue, int opcode, int immidiateAdditive, int immidiateScaling, int dataSourceA, int dataSourceB) {
            if (immidiateAdditive < -1048576 || immidiateAdditive > 1048575) { throw new IllegalArgumentException("immidiateAdditive must be between [-1048576..=1048575] inclusive, instead got " + immidiateAdditive); }
            if (immidiateScaling < 0 || immidiateScaling > 255) { throw new IllegalArgumentException("immidiateScaling must be between [0..=255] inclusive, instead got " + immidiateScaling); }
            return (slotEnabled ? 1L : 0L) | 
                (((long) nextSlotAction) << 1) | 
                ((invertValue ? 1L : 0L) << 3) | 
                (((long) opcode) << 4) | 
                (((long) immidiateAdditive) << 11) | 
                (((long) immidiateScaling) << 32) | 
                (((long) dataSourceA) << 40) | 
                (((long) dataSourceB) << 44);
        }

        /**
         * Constructs a DIGOUT1_CONFIG_9 setting.
         * 
         * @param slotEnabled Enable the digout slot (bool)
         * @param nextSlotAction How the digout slot interacts with the next slot (enum:NEXT_SLOT_ACTION)
         * @param invertValue Invert the digout slot's boolean value (bool)
         * @param opcode Opcode (enum:SLOT_OPCODE)
         * @param immidiateAdditive Additive immidiate (sint:21)
         * @param immidiateScaling Scaling immidiate (uint:8)
         * @param dataSourceA First ``a`` data source (enum:DATA_SOURCE)
         * @param dataSourceB Second ``b`` data source (enum:DATA_SOURCE)
         * @return message data as long
         */
        public static long constructDigout1Config9(boolean slotEnabled, int nextSlotAction, boolean invertValue, int opcode, int immidiateAdditive, int immidiateScaling, int dataSourceA, int dataSourceB) {
            if (immidiateAdditive < -1048576 || immidiateAdditive > 1048575) { throw new IllegalArgumentException("immidiateAdditive must be between [-1048576..=1048575] inclusive, instead got " + immidiateAdditive); }
            if (immidiateScaling < 0 || immidiateScaling > 255) { throw new IllegalArgumentException("immidiateScaling must be between [0..=255] inclusive, instead got " + immidiateScaling); }
            return (slotEnabled ? 1L : 0L) | 
                (((long) nextSlotAction) << 1) | 
                ((invertValue ? 1L : 0L) << 3) | 
                (((long) opcode) << 4) | 
                (((long) immidiateAdditive) << 11) | 
                (((long) immidiateScaling) << 32) | 
                (((long) dataSourceA) << 40) | 
                (((long) dataSourceB) << 44);
        }

        /**
         * Constructs a DIGOUT1_CONFIG_10 setting.
         * 
         * @param slotEnabled Enable the digout slot (bool)
         * @param nextSlotAction How the digout slot interacts with the next slot (enum:NEXT_SLOT_ACTION)
         * @param invertValue Invert the digout slot's boolean value (bool)
         * @param opcode Opcode (enum:SLOT_OPCODE)
         * @param immidiateAdditive Additive immidiate (sint:21)
         * @param immidiateScaling Scaling immidiate (uint:8)
         * @param dataSourceA First ``a`` data source (enum:DATA_SOURCE)
         * @param dataSourceB Second ``b`` data source (enum:DATA_SOURCE)
         * @return message data as long
         */
        public static long constructDigout1Config10(boolean slotEnabled, int nextSlotAction, boolean invertValue, int opcode, int immidiateAdditive, int immidiateScaling, int dataSourceA, int dataSourceB) {
            if (immidiateAdditive < -1048576 || immidiateAdditive > 1048575) { throw new IllegalArgumentException("immidiateAdditive must be between [-1048576..=1048575] inclusive, instead got " + immidiateAdditive); }
            if (immidiateScaling < 0 || immidiateScaling > 255) { throw new IllegalArgumentException("immidiateScaling must be between [0..=255] inclusive, instead got " + immidiateScaling); }
            return (slotEnabled ? 1L : 0L) | 
                (((long) nextSlotAction) << 1) | 
                ((invertValue ? 1L : 0L) << 3) | 
                (((long) opcode) << 4) | 
                (((long) immidiateAdditive) << 11) | 
                (((long) immidiateScaling) << 32) | 
                (((long) dataSourceA) << 40) | 
                (((long) dataSourceB) << 44);
        }

        /**
         * Constructs a DIGOUT1_CONFIG_11 setting.
         * 
         * @param slotEnabled Enable the digout slot (bool)
         * @param nextSlotAction How the digout slot interacts with the next slot (enum:NEXT_SLOT_ACTION)
         * @param invertValue Invert the digout slot's boolean value (bool)
         * @param opcode Opcode (enum:SLOT_OPCODE)
         * @param immidiateAdditive Additive immidiate (sint:21)
         * @param immidiateScaling Scaling immidiate (uint:8)
         * @param dataSourceA First ``a`` data source (enum:DATA_SOURCE)
         * @param dataSourceB Second ``b`` data source (enum:DATA_SOURCE)
         * @return message data as long
         */
        public static long constructDigout1Config11(boolean slotEnabled, int nextSlotAction, boolean invertValue, int opcode, int immidiateAdditive, int immidiateScaling, int dataSourceA, int dataSourceB) {
            if (immidiateAdditive < -1048576 || immidiateAdditive > 1048575) { throw new IllegalArgumentException("immidiateAdditive must be between [-1048576..=1048575] inclusive, instead got " + immidiateAdditive); }
            if (immidiateScaling < 0 || immidiateScaling > 255) { throw new IllegalArgumentException("immidiateScaling must be between [0..=255] inclusive, instead got " + immidiateScaling); }
            return (slotEnabled ? 1L : 0L) | 
                (((long) nextSlotAction) << 1) | 
                ((invertValue ? 1L : 0L) << 3) | 
                (((long) opcode) << 4) | 
                (((long) immidiateAdditive) << 11) | 
                (((long) immidiateScaling) << 32) | 
                (((long) dataSourceA) << 40) | 
                (((long) dataSourceB) << 44);
        }

        /**
         * Constructs a DIGOUT1_CONFIG_12 setting.
         * 
         * @param slotEnabled Enable the digout slot (bool)
         * @param nextSlotAction How the digout slot interacts with the next slot (enum:NEXT_SLOT_ACTION)
         * @param invertValue Invert the digout slot's boolean value (bool)
         * @param opcode Opcode (enum:SLOT_OPCODE)
         * @param immidiateAdditive Additive immidiate (sint:21)
         * @param immidiateScaling Scaling immidiate (uint:8)
         * @param dataSourceA First ``a`` data source (enum:DATA_SOURCE)
         * @param dataSourceB Second ``b`` data source (enum:DATA_SOURCE)
         * @return message data as long
         */
        public static long constructDigout1Config12(boolean slotEnabled, int nextSlotAction, boolean invertValue, int opcode, int immidiateAdditive, int immidiateScaling, int dataSourceA, int dataSourceB) {
            if (immidiateAdditive < -1048576 || immidiateAdditive > 1048575) { throw new IllegalArgumentException("immidiateAdditive must be between [-1048576..=1048575] inclusive, instead got " + immidiateAdditive); }
            if (immidiateScaling < 0 || immidiateScaling > 255) { throw new IllegalArgumentException("immidiateScaling must be between [0..=255] inclusive, instead got " + immidiateScaling); }
            return (slotEnabled ? 1L : 0L) | 
                (((long) nextSlotAction) << 1) | 
                ((invertValue ? 1L : 0L) << 3) | 
                (((long) opcode) << 4) | 
                (((long) immidiateAdditive) << 11) | 
                (((long) immidiateScaling) << 32) | 
                (((long) dataSourceA) << 40) | 
                (((long) dataSourceB) << 44);
        }

        /**
         * Constructs a DIGOUT1_CONFIG_13 setting.
         * 
         * @param slotEnabled Enable the digout slot (bool)
         * @param nextSlotAction How the digout slot interacts with the next slot (enum:NEXT_SLOT_ACTION)
         * @param invertValue Invert the digout slot's boolean value (bool)
         * @param opcode Opcode (enum:SLOT_OPCODE)
         * @param immidiateAdditive Additive immidiate (sint:21)
         * @param immidiateScaling Scaling immidiate (uint:8)
         * @param dataSourceA First ``a`` data source (enum:DATA_SOURCE)
         * @param dataSourceB Second ``b`` data source (enum:DATA_SOURCE)
         * @return message data as long
         */
        public static long constructDigout1Config13(boolean slotEnabled, int nextSlotAction, boolean invertValue, int opcode, int immidiateAdditive, int immidiateScaling, int dataSourceA, int dataSourceB) {
            if (immidiateAdditive < -1048576 || immidiateAdditive > 1048575) { throw new IllegalArgumentException("immidiateAdditive must be between [-1048576..=1048575] inclusive, instead got " + immidiateAdditive); }
            if (immidiateScaling < 0 || immidiateScaling > 255) { throw new IllegalArgumentException("immidiateScaling must be between [0..=255] inclusive, instead got " + immidiateScaling); }
            return (slotEnabled ? 1L : 0L) | 
                (((long) nextSlotAction) << 1) | 
                ((invertValue ? 1L : 0L) << 3) | 
                (((long) opcode) << 4) | 
                (((long) immidiateAdditive) << 11) | 
                (((long) immidiateScaling) << 32) | 
                (((long) dataSourceA) << 40) | 
                (((long) dataSourceB) << 44);
        }

        /**
         * Constructs a DIGOUT1_CONFIG_14 setting.
         * 
         * @param slotEnabled Enable the digout slot (bool)
         * @param nextSlotAction How the digout slot interacts with the next slot (enum:NEXT_SLOT_ACTION)
         * @param invertValue Invert the digout slot's boolean value (bool)
         * @param opcode Opcode (enum:SLOT_OPCODE)
         * @param immidiateAdditive Additive immidiate (sint:21)
         * @param immidiateScaling Scaling immidiate (uint:8)
         * @param dataSourceA First ``a`` data source (enum:DATA_SOURCE)
         * @param dataSourceB Second ``b`` data source (enum:DATA_SOURCE)
         * @return message data as long
         */
        public static long constructDigout1Config14(boolean slotEnabled, int nextSlotAction, boolean invertValue, int opcode, int immidiateAdditive, int immidiateScaling, int dataSourceA, int dataSourceB) {
            if (immidiateAdditive < -1048576 || immidiateAdditive > 1048575) { throw new IllegalArgumentException("immidiateAdditive must be between [-1048576..=1048575] inclusive, instead got " + immidiateAdditive); }
            if (immidiateScaling < 0 || immidiateScaling > 255) { throw new IllegalArgumentException("immidiateScaling must be between [0..=255] inclusive, instead got " + immidiateScaling); }
            return (slotEnabled ? 1L : 0L) | 
                (((long) nextSlotAction) << 1) | 
                ((invertValue ? 1L : 0L) << 3) | 
                (((long) opcode) << 4) | 
                (((long) immidiateAdditive) << 11) | 
                (((long) immidiateScaling) << 32) | 
                (((long) dataSourceA) << 40) | 
                (((long) dataSourceB) << 44);
        }

        /**
         * Constructs a DIGOUT1_CONFIG_15 setting.
         * 
         * @param slotEnabled Enable the digout slot (bool)
         * @param nextSlotAction How the digout slot interacts with the next slot (enum:NEXT_SLOT_ACTION)
         * @param invertValue Invert the digout slot's boolean value (bool)
         * @param opcode Opcode (enum:SLOT_OPCODE)
         * @param immidiateAdditive Additive immidiate (sint:21)
         * @param immidiateScaling Scaling immidiate (uint:8)
         * @param dataSourceA First ``a`` data source (enum:DATA_SOURCE)
         * @param dataSourceB Second ``b`` data source (enum:DATA_SOURCE)
         * @return message data as long
         */
        public static long constructDigout1Config15(boolean slotEnabled, int nextSlotAction, boolean invertValue, int opcode, int immidiateAdditive, int immidiateScaling, int dataSourceA, int dataSourceB) {
            if (immidiateAdditive < -1048576 || immidiateAdditive > 1048575) { throw new IllegalArgumentException("immidiateAdditive must be between [-1048576..=1048575] inclusive, instead got " + immidiateAdditive); }
            if (immidiateScaling < 0 || immidiateScaling > 255) { throw new IllegalArgumentException("immidiateScaling must be between [0..=255] inclusive, instead got " + immidiateScaling); }
            return (slotEnabled ? 1L : 0L) | 
                (((long) nextSlotAction) << 1) | 
                ((invertValue ? 1L : 0L) << 3) | 
                (((long) opcode) << 4) | 
                (((long) immidiateAdditive) << 11) | 
                (((long) immidiateScaling) << 32) | 
                (((long) dataSourceA) << 40) | 
                (((long) dataSourceB) << 44);
        }

        /**
         * Constructs a DIGOUT2_CONFIG_0 setting.
         * 
         * @param slotEnabled Enable the digout slot (bool)
         * @param nextSlotAction How the digout slot interacts with the next slot (enum:NEXT_SLOT_ACTION)
         * @param invertValue Invert the digout slot's boolean value (bool)
         * @param opcode Opcode (enum:SLOT_OPCODE)
         * @param immidiateAdditive Additive immidiate (sint:21)
         * @param immidiateScaling Scaling immidiate (uint:8)
         * @param dataSourceA First ``a`` data source (enum:DATA_SOURCE)
         * @param dataSourceB Second ``b`` data source (enum:DATA_SOURCE)
         * @return message data as long
         */
        public static long constructDigout2Config0(boolean slotEnabled, int nextSlotAction, boolean invertValue, int opcode, int immidiateAdditive, int immidiateScaling, int dataSourceA, int dataSourceB) {
            if (immidiateAdditive < -1048576 || immidiateAdditive > 1048575) { throw new IllegalArgumentException("immidiateAdditive must be between [-1048576..=1048575] inclusive, instead got " + immidiateAdditive); }
            if (immidiateScaling < 0 || immidiateScaling > 255) { throw new IllegalArgumentException("immidiateScaling must be between [0..=255] inclusive, instead got " + immidiateScaling); }
            return (slotEnabled ? 1L : 0L) | 
                (((long) nextSlotAction) << 1) | 
                ((invertValue ? 1L : 0L) << 3) | 
                (((long) opcode) << 4) | 
                (((long) immidiateAdditive) << 11) | 
                (((long) immidiateScaling) << 32) | 
                (((long) dataSourceA) << 40) | 
                (((long) dataSourceB) << 44);
        }

        /**
         * Constructs a DIGOUT2_CONFIG_1 setting.
         * 
         * @param slotEnabled Enable the digout slot (bool)
         * @param nextSlotAction How the digout slot interacts with the next slot (enum:NEXT_SLOT_ACTION)
         * @param invertValue Invert the digout slot's boolean value (bool)
         * @param opcode Opcode (enum:SLOT_OPCODE)
         * @param immidiateAdditive Additive immidiate (sint:21)
         * @param immidiateScaling Scaling immidiate (uint:8)
         * @param dataSourceA First ``a`` data source (enum:DATA_SOURCE)
         * @param dataSourceB Second ``b`` data source (enum:DATA_SOURCE)
         * @return message data as long
         */
        public static long constructDigout2Config1(boolean slotEnabled, int nextSlotAction, boolean invertValue, int opcode, int immidiateAdditive, int immidiateScaling, int dataSourceA, int dataSourceB) {
            if (immidiateAdditive < -1048576 || immidiateAdditive > 1048575) { throw new IllegalArgumentException("immidiateAdditive must be between [-1048576..=1048575] inclusive, instead got " + immidiateAdditive); }
            if (immidiateScaling < 0 || immidiateScaling > 255) { throw new IllegalArgumentException("immidiateScaling must be between [0..=255] inclusive, instead got " + immidiateScaling); }
            return (slotEnabled ? 1L : 0L) | 
                (((long) nextSlotAction) << 1) | 
                ((invertValue ? 1L : 0L) << 3) | 
                (((long) opcode) << 4) | 
                (((long) immidiateAdditive) << 11) | 
                (((long) immidiateScaling) << 32) | 
                (((long) dataSourceA) << 40) | 
                (((long) dataSourceB) << 44);
        }

        /**
         * Constructs a DIGOUT2_CONFIG_2 setting.
         * 
         * @param slotEnabled Enable the digout slot (bool)
         * @param nextSlotAction How the digout slot interacts with the next slot (enum:NEXT_SLOT_ACTION)
         * @param invertValue Invert the digout slot's boolean value (bool)
         * @param opcode Opcode (enum:SLOT_OPCODE)
         * @param immidiateAdditive Additive immidiate (sint:21)
         * @param immidiateScaling Scaling immidiate (uint:8)
         * @param dataSourceA First ``a`` data source (enum:DATA_SOURCE)
         * @param dataSourceB Second ``b`` data source (enum:DATA_SOURCE)
         * @return message data as long
         */
        public static long constructDigout2Config2(boolean slotEnabled, int nextSlotAction, boolean invertValue, int opcode, int immidiateAdditive, int immidiateScaling, int dataSourceA, int dataSourceB) {
            if (immidiateAdditive < -1048576 || immidiateAdditive > 1048575) { throw new IllegalArgumentException("immidiateAdditive must be between [-1048576..=1048575] inclusive, instead got " + immidiateAdditive); }
            if (immidiateScaling < 0 || immidiateScaling > 255) { throw new IllegalArgumentException("immidiateScaling must be between [0..=255] inclusive, instead got " + immidiateScaling); }
            return (slotEnabled ? 1L : 0L) | 
                (((long) nextSlotAction) << 1) | 
                ((invertValue ? 1L : 0L) << 3) | 
                (((long) opcode) << 4) | 
                (((long) immidiateAdditive) << 11) | 
                (((long) immidiateScaling) << 32) | 
                (((long) dataSourceA) << 40) | 
                (((long) dataSourceB) << 44);
        }

        /**
         * Constructs a DIGOUT2_CONFIG_3 setting.
         * 
         * @param slotEnabled Enable the digout slot (bool)
         * @param nextSlotAction How the digout slot interacts with the next slot (enum:NEXT_SLOT_ACTION)
         * @param invertValue Invert the digout slot's boolean value (bool)
         * @param opcode Opcode (enum:SLOT_OPCODE)
         * @param immidiateAdditive Additive immidiate (sint:21)
         * @param immidiateScaling Scaling immidiate (uint:8)
         * @param dataSourceA First ``a`` data source (enum:DATA_SOURCE)
         * @param dataSourceB Second ``b`` data source (enum:DATA_SOURCE)
         * @return message data as long
         */
        public static long constructDigout2Config3(boolean slotEnabled, int nextSlotAction, boolean invertValue, int opcode, int immidiateAdditive, int immidiateScaling, int dataSourceA, int dataSourceB) {
            if (immidiateAdditive < -1048576 || immidiateAdditive > 1048575) { throw new IllegalArgumentException("immidiateAdditive must be between [-1048576..=1048575] inclusive, instead got " + immidiateAdditive); }
            if (immidiateScaling < 0 || immidiateScaling > 255) { throw new IllegalArgumentException("immidiateScaling must be between [0..=255] inclusive, instead got " + immidiateScaling); }
            return (slotEnabled ? 1L : 0L) | 
                (((long) nextSlotAction) << 1) | 
                ((invertValue ? 1L : 0L) << 3) | 
                (((long) opcode) << 4) | 
                (((long) immidiateAdditive) << 11) | 
                (((long) immidiateScaling) << 32) | 
                (((long) dataSourceA) << 40) | 
                (((long) dataSourceB) << 44);
        }

        /**
         * Constructs a DIGOUT2_CONFIG_4 setting.
         * 
         * @param slotEnabled Enable the digout slot (bool)
         * @param nextSlotAction How the digout slot interacts with the next slot (enum:NEXT_SLOT_ACTION)
         * @param invertValue Invert the digout slot's boolean value (bool)
         * @param opcode Opcode (enum:SLOT_OPCODE)
         * @param immidiateAdditive Additive immidiate (sint:21)
         * @param immidiateScaling Scaling immidiate (uint:8)
         * @param dataSourceA First ``a`` data source (enum:DATA_SOURCE)
         * @param dataSourceB Second ``b`` data source (enum:DATA_SOURCE)
         * @return message data as long
         */
        public static long constructDigout2Config4(boolean slotEnabled, int nextSlotAction, boolean invertValue, int opcode, int immidiateAdditive, int immidiateScaling, int dataSourceA, int dataSourceB) {
            if (immidiateAdditive < -1048576 || immidiateAdditive > 1048575) { throw new IllegalArgumentException("immidiateAdditive must be between [-1048576..=1048575] inclusive, instead got " + immidiateAdditive); }
            if (immidiateScaling < 0 || immidiateScaling > 255) { throw new IllegalArgumentException("immidiateScaling must be between [0..=255] inclusive, instead got " + immidiateScaling); }
            return (slotEnabled ? 1L : 0L) | 
                (((long) nextSlotAction) << 1) | 
                ((invertValue ? 1L : 0L) << 3) | 
                (((long) opcode) << 4) | 
                (((long) immidiateAdditive) << 11) | 
                (((long) immidiateScaling) << 32) | 
                (((long) dataSourceA) << 40) | 
                (((long) dataSourceB) << 44);
        }

        /**
         * Constructs a DIGOUT2_CONFIG_5 setting.
         * 
         * @param slotEnabled Enable the digout slot (bool)
         * @param nextSlotAction How the digout slot interacts with the next slot (enum:NEXT_SLOT_ACTION)
         * @param invertValue Invert the digout slot's boolean value (bool)
         * @param opcode Opcode (enum:SLOT_OPCODE)
         * @param immidiateAdditive Additive immidiate (sint:21)
         * @param immidiateScaling Scaling immidiate (uint:8)
         * @param dataSourceA First ``a`` data source (enum:DATA_SOURCE)
         * @param dataSourceB Second ``b`` data source (enum:DATA_SOURCE)
         * @return message data as long
         */
        public static long constructDigout2Config5(boolean slotEnabled, int nextSlotAction, boolean invertValue, int opcode, int immidiateAdditive, int immidiateScaling, int dataSourceA, int dataSourceB) {
            if (immidiateAdditive < -1048576 || immidiateAdditive > 1048575) { throw new IllegalArgumentException("immidiateAdditive must be between [-1048576..=1048575] inclusive, instead got " + immidiateAdditive); }
            if (immidiateScaling < 0 || immidiateScaling > 255) { throw new IllegalArgumentException("immidiateScaling must be between [0..=255] inclusive, instead got " + immidiateScaling); }
            return (slotEnabled ? 1L : 0L) | 
                (((long) nextSlotAction) << 1) | 
                ((invertValue ? 1L : 0L) << 3) | 
                (((long) opcode) << 4) | 
                (((long) immidiateAdditive) << 11) | 
                (((long) immidiateScaling) << 32) | 
                (((long) dataSourceA) << 40) | 
                (((long) dataSourceB) << 44);
        }

        /**
         * Constructs a DIGOUT2_CONFIG_6 setting.
         * 
         * @param slotEnabled Enable the digout slot (bool)
         * @param nextSlotAction How the digout slot interacts with the next slot (enum:NEXT_SLOT_ACTION)
         * @param invertValue Invert the digout slot's boolean value (bool)
         * @param opcode Opcode (enum:SLOT_OPCODE)
         * @param immidiateAdditive Additive immidiate (sint:21)
         * @param immidiateScaling Scaling immidiate (uint:8)
         * @param dataSourceA First ``a`` data source (enum:DATA_SOURCE)
         * @param dataSourceB Second ``b`` data source (enum:DATA_SOURCE)
         * @return message data as long
         */
        public static long constructDigout2Config6(boolean slotEnabled, int nextSlotAction, boolean invertValue, int opcode, int immidiateAdditive, int immidiateScaling, int dataSourceA, int dataSourceB) {
            if (immidiateAdditive < -1048576 || immidiateAdditive > 1048575) { throw new IllegalArgumentException("immidiateAdditive must be between [-1048576..=1048575] inclusive, instead got " + immidiateAdditive); }
            if (immidiateScaling < 0 || immidiateScaling > 255) { throw new IllegalArgumentException("immidiateScaling must be between [0..=255] inclusive, instead got " + immidiateScaling); }
            return (slotEnabled ? 1L : 0L) | 
                (((long) nextSlotAction) << 1) | 
                ((invertValue ? 1L : 0L) << 3) | 
                (((long) opcode) << 4) | 
                (((long) immidiateAdditive) << 11) | 
                (((long) immidiateScaling) << 32) | 
                (((long) dataSourceA) << 40) | 
                (((long) dataSourceB) << 44);
        }

        /**
         * Constructs a DIGOUT2_CONFIG_7 setting.
         * 
         * @param slotEnabled Enable the digout slot (bool)
         * @param nextSlotAction How the digout slot interacts with the next slot (enum:NEXT_SLOT_ACTION)
         * @param invertValue Invert the digout slot's boolean value (bool)
         * @param opcode Opcode (enum:SLOT_OPCODE)
         * @param immidiateAdditive Additive immidiate (sint:21)
         * @param immidiateScaling Scaling immidiate (uint:8)
         * @param dataSourceA First ``a`` data source (enum:DATA_SOURCE)
         * @param dataSourceB Second ``b`` data source (enum:DATA_SOURCE)
         * @return message data as long
         */
        public static long constructDigout2Config7(boolean slotEnabled, int nextSlotAction, boolean invertValue, int opcode, int immidiateAdditive, int immidiateScaling, int dataSourceA, int dataSourceB) {
            if (immidiateAdditive < -1048576 || immidiateAdditive > 1048575) { throw new IllegalArgumentException("immidiateAdditive must be between [-1048576..=1048575] inclusive, instead got " + immidiateAdditive); }
            if (immidiateScaling < 0 || immidiateScaling > 255) { throw new IllegalArgumentException("immidiateScaling must be between [0..=255] inclusive, instead got " + immidiateScaling); }
            return (slotEnabled ? 1L : 0L) | 
                (((long) nextSlotAction) << 1) | 
                ((invertValue ? 1L : 0L) << 3) | 
                (((long) opcode) << 4) | 
                (((long) immidiateAdditive) << 11) | 
                (((long) immidiateScaling) << 32) | 
                (((long) dataSourceA) << 40) | 
                (((long) dataSourceB) << 44);
        }

        /**
         * Constructs a DIGOUT2_CONFIG_8 setting.
         * 
         * @param slotEnabled Enable the digout slot (bool)
         * @param nextSlotAction How the digout slot interacts with the next slot (enum:NEXT_SLOT_ACTION)
         * @param invertValue Invert the digout slot's boolean value (bool)
         * @param opcode Opcode (enum:SLOT_OPCODE)
         * @param immidiateAdditive Additive immidiate (sint:21)
         * @param immidiateScaling Scaling immidiate (uint:8)
         * @param dataSourceA First ``a`` data source (enum:DATA_SOURCE)
         * @param dataSourceB Second ``b`` data source (enum:DATA_SOURCE)
         * @return message data as long
         */
        public static long constructDigout2Config8(boolean slotEnabled, int nextSlotAction, boolean invertValue, int opcode, int immidiateAdditive, int immidiateScaling, int dataSourceA, int dataSourceB) {
            if (immidiateAdditive < -1048576 || immidiateAdditive > 1048575) { throw new IllegalArgumentException("immidiateAdditive must be between [-1048576..=1048575] inclusive, instead got " + immidiateAdditive); }
            if (immidiateScaling < 0 || immidiateScaling > 255) { throw new IllegalArgumentException("immidiateScaling must be between [0..=255] inclusive, instead got " + immidiateScaling); }
            return (slotEnabled ? 1L : 0L) | 
                (((long) nextSlotAction) << 1) | 
                ((invertValue ? 1L : 0L) << 3) | 
                (((long) opcode) << 4) | 
                (((long) immidiateAdditive) << 11) | 
                (((long) immidiateScaling) << 32) | 
                (((long) dataSourceA) << 40) | 
                (((long) dataSourceB) << 44);
        }

        /**
         * Constructs a DIGOUT2_CONFIG_9 setting.
         * 
         * @param slotEnabled Enable the digout slot (bool)
         * @param nextSlotAction How the digout slot interacts with the next slot (enum:NEXT_SLOT_ACTION)
         * @param invertValue Invert the digout slot's boolean value (bool)
         * @param opcode Opcode (enum:SLOT_OPCODE)
         * @param immidiateAdditive Additive immidiate (sint:21)
         * @param immidiateScaling Scaling immidiate (uint:8)
         * @param dataSourceA First ``a`` data source (enum:DATA_SOURCE)
         * @param dataSourceB Second ``b`` data source (enum:DATA_SOURCE)
         * @return message data as long
         */
        public static long constructDigout2Config9(boolean slotEnabled, int nextSlotAction, boolean invertValue, int opcode, int immidiateAdditive, int immidiateScaling, int dataSourceA, int dataSourceB) {
            if (immidiateAdditive < -1048576 || immidiateAdditive > 1048575) { throw new IllegalArgumentException("immidiateAdditive must be between [-1048576..=1048575] inclusive, instead got " + immidiateAdditive); }
            if (immidiateScaling < 0 || immidiateScaling > 255) { throw new IllegalArgumentException("immidiateScaling must be between [0..=255] inclusive, instead got " + immidiateScaling); }
            return (slotEnabled ? 1L : 0L) | 
                (((long) nextSlotAction) << 1) | 
                ((invertValue ? 1L : 0L) << 3) | 
                (((long) opcode) << 4) | 
                (((long) immidiateAdditive) << 11) | 
                (((long) immidiateScaling) << 32) | 
                (((long) dataSourceA) << 40) | 
                (((long) dataSourceB) << 44);
        }

        /**
         * Constructs a DIGOUT2_CONFIG_10 setting.
         * 
         * @param slotEnabled Enable the digout slot (bool)
         * @param nextSlotAction How the digout slot interacts with the next slot (enum:NEXT_SLOT_ACTION)
         * @param invertValue Invert the digout slot's boolean value (bool)
         * @param opcode Opcode (enum:SLOT_OPCODE)
         * @param immidiateAdditive Additive immidiate (sint:21)
         * @param immidiateScaling Scaling immidiate (uint:8)
         * @param dataSourceA First ``a`` data source (enum:DATA_SOURCE)
         * @param dataSourceB Second ``b`` data source (enum:DATA_SOURCE)
         * @return message data as long
         */
        public static long constructDigout2Config10(boolean slotEnabled, int nextSlotAction, boolean invertValue, int opcode, int immidiateAdditive, int immidiateScaling, int dataSourceA, int dataSourceB) {
            if (immidiateAdditive < -1048576 || immidiateAdditive > 1048575) { throw new IllegalArgumentException("immidiateAdditive must be between [-1048576..=1048575] inclusive, instead got " + immidiateAdditive); }
            if (immidiateScaling < 0 || immidiateScaling > 255) { throw new IllegalArgumentException("immidiateScaling must be between [0..=255] inclusive, instead got " + immidiateScaling); }
            return (slotEnabled ? 1L : 0L) | 
                (((long) nextSlotAction) << 1) | 
                ((invertValue ? 1L : 0L) << 3) | 
                (((long) opcode) << 4) | 
                (((long) immidiateAdditive) << 11) | 
                (((long) immidiateScaling) << 32) | 
                (((long) dataSourceA) << 40) | 
                (((long) dataSourceB) << 44);
        }

        /**
         * Constructs a DIGOUT2_CONFIG_11 setting.
         * 
         * @param slotEnabled Enable the digout slot (bool)
         * @param nextSlotAction How the digout slot interacts with the next slot (enum:NEXT_SLOT_ACTION)
         * @param invertValue Invert the digout slot's boolean value (bool)
         * @param opcode Opcode (enum:SLOT_OPCODE)
         * @param immidiateAdditive Additive immidiate (sint:21)
         * @param immidiateScaling Scaling immidiate (uint:8)
         * @param dataSourceA First ``a`` data source (enum:DATA_SOURCE)
         * @param dataSourceB Second ``b`` data source (enum:DATA_SOURCE)
         * @return message data as long
         */
        public static long constructDigout2Config11(boolean slotEnabled, int nextSlotAction, boolean invertValue, int opcode, int immidiateAdditive, int immidiateScaling, int dataSourceA, int dataSourceB) {
            if (immidiateAdditive < -1048576 || immidiateAdditive > 1048575) { throw new IllegalArgumentException("immidiateAdditive must be between [-1048576..=1048575] inclusive, instead got " + immidiateAdditive); }
            if (immidiateScaling < 0 || immidiateScaling > 255) { throw new IllegalArgumentException("immidiateScaling must be between [0..=255] inclusive, instead got " + immidiateScaling); }
            return (slotEnabled ? 1L : 0L) | 
                (((long) nextSlotAction) << 1) | 
                ((invertValue ? 1L : 0L) << 3) | 
                (((long) opcode) << 4) | 
                (((long) immidiateAdditive) << 11) | 
                (((long) immidiateScaling) << 32) | 
                (((long) dataSourceA) << 40) | 
                (((long) dataSourceB) << 44);
        }

        /**
         * Constructs a DIGOUT2_CONFIG_12 setting.
         * 
         * @param slotEnabled Enable the digout slot (bool)
         * @param nextSlotAction How the digout slot interacts with the next slot (enum:NEXT_SLOT_ACTION)
         * @param invertValue Invert the digout slot's boolean value (bool)
         * @param opcode Opcode (enum:SLOT_OPCODE)
         * @param immidiateAdditive Additive immidiate (sint:21)
         * @param immidiateScaling Scaling immidiate (uint:8)
         * @param dataSourceA First ``a`` data source (enum:DATA_SOURCE)
         * @param dataSourceB Second ``b`` data source (enum:DATA_SOURCE)
         * @return message data as long
         */
        public static long constructDigout2Config12(boolean slotEnabled, int nextSlotAction, boolean invertValue, int opcode, int immidiateAdditive, int immidiateScaling, int dataSourceA, int dataSourceB) {
            if (immidiateAdditive < -1048576 || immidiateAdditive > 1048575) { throw new IllegalArgumentException("immidiateAdditive must be between [-1048576..=1048575] inclusive, instead got " + immidiateAdditive); }
            if (immidiateScaling < 0 || immidiateScaling > 255) { throw new IllegalArgumentException("immidiateScaling must be between [0..=255] inclusive, instead got " + immidiateScaling); }
            return (slotEnabled ? 1L : 0L) | 
                (((long) nextSlotAction) << 1) | 
                ((invertValue ? 1L : 0L) << 3) | 
                (((long) opcode) << 4) | 
                (((long) immidiateAdditive) << 11) | 
                (((long) immidiateScaling) << 32) | 
                (((long) dataSourceA) << 40) | 
                (((long) dataSourceB) << 44);
        }

        /**
         * Constructs a DIGOUT2_CONFIG_13 setting.
         * 
         * @param slotEnabled Enable the digout slot (bool)
         * @param nextSlotAction How the digout slot interacts with the next slot (enum:NEXT_SLOT_ACTION)
         * @param invertValue Invert the digout slot's boolean value (bool)
         * @param opcode Opcode (enum:SLOT_OPCODE)
         * @param immidiateAdditive Additive immidiate (sint:21)
         * @param immidiateScaling Scaling immidiate (uint:8)
         * @param dataSourceA First ``a`` data source (enum:DATA_SOURCE)
         * @param dataSourceB Second ``b`` data source (enum:DATA_SOURCE)
         * @return message data as long
         */
        public static long constructDigout2Config13(boolean slotEnabled, int nextSlotAction, boolean invertValue, int opcode, int immidiateAdditive, int immidiateScaling, int dataSourceA, int dataSourceB) {
            if (immidiateAdditive < -1048576 || immidiateAdditive > 1048575) { throw new IllegalArgumentException("immidiateAdditive must be between [-1048576..=1048575] inclusive, instead got " + immidiateAdditive); }
            if (immidiateScaling < 0 || immidiateScaling > 255) { throw new IllegalArgumentException("immidiateScaling must be between [0..=255] inclusive, instead got " + immidiateScaling); }
            return (slotEnabled ? 1L : 0L) | 
                (((long) nextSlotAction) << 1) | 
                ((invertValue ? 1L : 0L) << 3) | 
                (((long) opcode) << 4) | 
                (((long) immidiateAdditive) << 11) | 
                (((long) immidiateScaling) << 32) | 
                (((long) dataSourceA) << 40) | 
                (((long) dataSourceB) << 44);
        }

        /**
         * Constructs a DIGOUT2_CONFIG_14 setting.
         * 
         * @param slotEnabled Enable the digout slot (bool)
         * @param nextSlotAction How the digout slot interacts with the next slot (enum:NEXT_SLOT_ACTION)
         * @param invertValue Invert the digout slot's boolean value (bool)
         * @param opcode Opcode (enum:SLOT_OPCODE)
         * @param immidiateAdditive Additive immidiate (sint:21)
         * @param immidiateScaling Scaling immidiate (uint:8)
         * @param dataSourceA First ``a`` data source (enum:DATA_SOURCE)
         * @param dataSourceB Second ``b`` data source (enum:DATA_SOURCE)
         * @return message data as long
         */
        public static long constructDigout2Config14(boolean slotEnabled, int nextSlotAction, boolean invertValue, int opcode, int immidiateAdditive, int immidiateScaling, int dataSourceA, int dataSourceB) {
            if (immidiateAdditive < -1048576 || immidiateAdditive > 1048575) { throw new IllegalArgumentException("immidiateAdditive must be between [-1048576..=1048575] inclusive, instead got " + immidiateAdditive); }
            if (immidiateScaling < 0 || immidiateScaling > 255) { throw new IllegalArgumentException("immidiateScaling must be between [0..=255] inclusive, instead got " + immidiateScaling); }
            return (slotEnabled ? 1L : 0L) | 
                (((long) nextSlotAction) << 1) | 
                ((invertValue ? 1L : 0L) << 3) | 
                (((long) opcode) << 4) | 
                (((long) immidiateAdditive) << 11) | 
                (((long) immidiateScaling) << 32) | 
                (((long) dataSourceA) << 40) | 
                (((long) dataSourceB) << 44);
        }

        /**
         * Constructs a DIGOUT2_CONFIG_15 setting.
         * 
         * @param slotEnabled Enable the digout slot (bool)
         * @param nextSlotAction How the digout slot interacts with the next slot (enum:NEXT_SLOT_ACTION)
         * @param invertValue Invert the digout slot's boolean value (bool)
         * @param opcode Opcode (enum:SLOT_OPCODE)
         * @param immidiateAdditive Additive immidiate (sint:21)
         * @param immidiateScaling Scaling immidiate (uint:8)
         * @param dataSourceA First ``a`` data source (enum:DATA_SOURCE)
         * @param dataSourceB Second ``b`` data source (enum:DATA_SOURCE)
         * @return message data as long
         */
        public static long constructDigout2Config15(boolean slotEnabled, int nextSlotAction, boolean invertValue, int opcode, int immidiateAdditive, int immidiateScaling, int dataSourceA, int dataSourceB) {
            if (immidiateAdditive < -1048576 || immidiateAdditive > 1048575) { throw new IllegalArgumentException("immidiateAdditive must be between [-1048576..=1048575] inclusive, instead got " + immidiateAdditive); }
            if (immidiateScaling < 0 || immidiateScaling > 255) { throw new IllegalArgumentException("immidiateScaling must be between [0..=255] inclusive, instead got " + immidiateScaling); }
            return (slotEnabled ? 1L : 0L) | 
                (((long) nextSlotAction) << 1) | 
                ((invertValue ? 1L : 0L) << 3) | 
                (((long) opcode) << 4) | 
                (((long) immidiateAdditive) << 11) | 
                (((long) immidiateScaling) << 32) | 
                (((long) dataSourceA) << 40) | 
                (((long) dataSourceB) << 44);
        }

        /**
         * Constructs a DEVICE_TYPE setting.
         * 
         * @param deviceType Device-specific type identifier (uint:16)
         * @return message data as long
         */
        public static long constructDeviceType(int deviceType) {
            if (deviceType < 0 || deviceType > 65535) { throw new IllegalArgumentException("deviceType must be between [0..=65535] inclusive, instead got " + deviceType); }
            return ((long) deviceType);
        }

        /**
         * Constructs a FIRMWARE_VERSION setting.
         * 
         * @param firmwarePatch Firmware version patch number (uint:8)
         * @param firmwareMinor Firmware version minor number (uint:8)
         * @param firmwareYear Firmware version year (uint:16)
         * @return message data as long
         */
        public static long constructFirmwareVersion(int firmwarePatch, int firmwareMinor, int firmwareYear) {
            if (firmwarePatch < 0 || firmwarePatch > 255) { throw new IllegalArgumentException("firmwarePatch must be between [0..=255] inclusive, instead got " + firmwarePatch); }
            if (firmwareMinor < 0 || firmwareMinor > 255) { throw new IllegalArgumentException("firmwareMinor must be between [0..=255] inclusive, instead got " + firmwareMinor); }
            if (firmwareYear < 0 || firmwareYear > 65535) { throw new IllegalArgumentException("firmwareYear must be between [0..=65535] inclusive, instead got " + firmwareYear); }
            return ((long) firmwarePatch) | 
                (((long) firmwareMinor) << 8) | 
                (((long) firmwareYear) << 16);
        }

        /**
         * Constructs a SERIAL_NUMBER setting.
         * 
         * @param serialNumber Serial number (buf:48)
         * @return message data as long
         */
        public static long constructSerialNumber(long serialNumber) {
            if (serialNumber < 0L || serialNumber > 281474976710655L) { throw new IllegalArgumentException("serialNumber must be between [0..=281474976710655] inclusive, instead got " + serialNumber); }
            return (serialNumber & 0xffffffffffffL);
        }

        /**
         * Constructs a STATUS_FRAME_PERIOD setting.
         * 
         * @param statusFramePeriod Status frame period (ms) (uint:16)
         * @return message data as long
         */
        public static long constructStatusFramePeriod(int statusFramePeriod) {
            if (statusFramePeriod < 1 || statusFramePeriod > 16383) { throw new IllegalArgumentException("statusFramePeriod must be between [1..=16383] inclusive, instead got " + statusFramePeriod); }
            return ((long) statusFramePeriod);
        }

        /** List of settings to fetch for. */
        public static int settingsAddresses[] = {
            kStatusFramePeriod,
            kDistanceFramePeriod,
            kColorFramePeriod,
            kDigoutFramePeriod,
            kDistanceExtraFrameMode,
            kColorExtraFrameMode,
            kLampBrightness,
            kColorIntegrationPeriod,
            kDistanceIntegrationPeriod,
            kDigout1OutputConfig,
            kDigout2OutputConfig,
            kDigout1MessageOnChange,
            kDigout2MessageOnChange,
        };

        /** Creates a HashMap of writable default settings. 
         * @return default settings
         */
        public static Map<Integer, Long> defaultSettings() {
            Map<Integer, Long> stg = new HashMap<>();
            stg.put(kStatusFramePeriod, 0x64L);
            stg.put(kDistanceFramePeriod, 0x14L);
            stg.put(kColorFramePeriod, 0x19L);
            stg.put(kDigoutFramePeriod, 0x64L);
            stg.put(kDistanceExtraFrameMode, 0x1L);
            stg.put(kColorExtraFrameMode, 0x1L);
            stg.put(kLampBrightness, 0x8ca0L);
            stg.put(kColorIntegrationPeriod, 0x4L);
            stg.put(kDistanceIntegrationPeriod, 0x6L);
            stg.put(kDigout1OutputConfig, 0x0L);
            stg.put(kDigout2OutputConfig, 0x0L);
            stg.put(kDigout1MessageOnChange, 0x0L);
            stg.put(kDigout2MessageOnChange, 0x0L);
            return stg;
        }

    }

    /**
     * Canandcolor enums.
     */
    public static class Enums {
        private Enums() {}

        /**
         * enum Canandcolor::EXTRA_FRAME_MODE.
         */
        public static class ExtraFrameMode {
            private ExtraFrameMode() {}

            /**
             * Do not emit extra frames beyond those specified in the frame period
             */
            public static final int kDisabled              = 0x0;

            /**
             * Transmits a frame immidiately once readings change
             */
            public static final int kEarlyTransmitOnChange = 0x1;
        }

        /**
         * enum Canandcolor::DIGOUT_OUTPUT_CONFIG.
         */
        public static class DigoutOutputConfig {
            private DigoutOutputConfig() {}

            /**
             * Disable output on this pin
             */
            public static final int kDisabled              = 0x0;

            /**
             * Use digital logic pin, output 3.3v on true and 0v on false
             */
            public static final int kDigoutLogicActiveHigh = 0x1;

            /**
             * Use digital logic pin, output 0v on true and 3.3v on false
             */
            public static final int kDigoutLogicActiveLow  = 0x2;

            /**
             * Output a duty cycle on this pin. Only works on OUT2
             */
            public static final int kDutyCycleOutput       = 0x3;
        }

        /**
         * enum Canandcolor::SLOT_OPCODE.
         */
        public static class SlotOpcode {
            private SlotOpcode() {}

            /**
             * true if ``a = b * (imm_scaling + 1) / 256 + (imm_additive)``
             */
            public static final int kEquals              = 0x0;

            /**
             * true if ``a &lt; b * (imm_scaling + 1) / 256 + (imm_additive)``
             */
            public static final int kLessThan            = 0x1;

            /**
             * true if ``a &gt; b * (imm_scaling + 1) / 256 + (imm_additive)``
             */
            public static final int kGreaterThan         = 0x2;

            /**
             * true if ``a &lt;= b * (imm_scaling + 1) / 256 + (imm_additive)``
             */
            public static final int kLessThanOrEquals    = 0x3;

            /**
             * true if``a &gt;= b * (imm_scaling + 1) / 256 + (imm_additive)``
             */
            public static final int kGreaterThanOrEquals = 0x4;

            /**
             * true if previous slot true for ``b * (imm_scaling + 1) / 256 + (imm_additive)`` milliseconds
             */
            public static final int kPrevSlotTrue        = 0x5;

            /**
             * true if previous joined-slot-chain true for ``b * (imm_scaling + 1) / 256 + (imm_additive)`` milliseconds
             */
            public static final int kPrevClauseTrue      = 0x6;
        }

        /**
         * enum Canandcolor::NEXT_SLOT_ACTION.
         */
        public static class NextSlotAction {
            private NextSlotAction() {}

            /**
             * Do not interact with the next slot
             */
            public static final int kTerminateChain  = 0x0;

            /**
             * Logical OR with next slot
             */
            public static final int kOrWithNextSlot  = 0x1;

            /**
             * Logical XOR with next slot
             */
            public static final int kXorWithNextSlot = 0x2;

            /**
             * Logical AND with next slot
             */
            public static final int kAndWithNextSlot = 0x3;
        }

        /**
         * enum Canandcolor::DATA_SOURCE.
         */
        public static class DataSource {
            private DataSource() {}

            /**
             * Always reads zero; can be used to compare only to additive immidate0
             */
            public static final int kZero       = 0x0;

            /**
             * Distance reading
             */
            public static final int kDistance   = 0x1;

            /**
             * Red reading
             */
            public static final int kRed        = 0x2;

            /**
             * Green reading
             */
            public static final int kGreen      = 0x3;

            /**
             * Blue reading
             */
            public static final int kBlue       = 0x4;

            /**
             * Hue reading
             */
            public static final int kHue        = 0x5;

            /**
             * Saturation reading
             */
            public static final int kSaturation = 0x6;

            /**
             * Value reading
             */
            public static final int kValue      = 0x7;
        }

        /**
         * enum Canandcolor::COLOR_INTEGRATION_PERIOD.
         */
        public static class ColorIntegrationPeriod {
            private ColorIntegrationPeriod() {}

            /**
             * 400 ms - 20 bit resolution
             */
            public static final int kPeriod400MsResolution20Bit = 0x0;

            /**
             * 200 ms - 19 bit resolution
             */
            public static final int kPeriod200MsResolution19Bit = 0x1;

            /**
             * 100 ms - 18 bit resolution
             */
            public static final int kPeriod100MsResolution18Bit = 0x2;

            /**
             * 50 ms - 17 bit resolution
             */
            public static final int kPeriod50MsResolution17Bit  = 0x3;

            /**
             * 25 ms - 16 bit resolution
             */
            public static final int kPeriod25MsResolution16Bit  = 0x4;
        }

        /**
         * enum Canandcolor::DISTANCE_INTEGRATION_PERIOD.
         */
        public static class DistanceIntegrationPeriod {
            private DistanceIntegrationPeriod() {}

            /**
             * 5 millisecond period
             */
            public static final int kPeriod5Ms    = 0x0;

            /**
             * 7.5 millisecond period
             */
            public static final int kPeriod7p5Ms  = 0x1;

            /**
             * 10 millisecond period
             */
            public static final int kPeriod10Ms   = 0x2;

            /**
             * 12.5 millisecond period
             */
            public static final int kPeriod12p5Ms = 0x3;

            /**
             * 15 millisecond period
             */
            public static final int kPeriod15Ms   = 0x4;

            /**
             * 17.5 millisecond period
             */
            public static final int kPeriod17p5Ms = 0x5;

            /**
             * 20 millisecond period
             */
            public static final int kPeriod20Ms   = 0x6;

            /**
             * 40 millisecond period
             */
            public static final int kPeriod40Ms   = 0x7;
        }

        /**
         * enum Canandcolor::SETTING_COMMAND.
         */
        public static class SettingCommand {
            private SettingCommand() {}

            /**
             * Fetch all settings from device via a series of :ref:`report setting&lt;msg_report_setting&gt;` messages of all indexes
             */
            public static final int kFetchSettings       = 0x0;

            /**
             * Reset all resettanble settings to factory default, and broadcast all setting values via
             * :ref:`report setting&lt;msg_report_setting&gt;` messages.
             */
            public static final int kResetFactoryDefault = 0x1;

            /**
             * Requests to fetch a single setting from device, with its value reported via the 
             * :ref:`report setting&lt;msg_report_setting&gt;` message. 
             * 
             * This requires the use of the second byte to specify the setting index to fetch.
             */
            public static final int kFetchSettingValue   = 0x2;

            /**
             * Clear all digout1 slots
             */
            public static final int kClearDigout1        = 0xff;

            /**
             * Clear all digout2 slots
             */
            public static final int kClearDigout2        = 0xfe;

            /**
             * Fetch all digout1 slots and settings
             */
            public static final int kFetchDigout1        = 0xfd;

            /**
             * Fetch all digout2 slots and settings
             */
            public static final int kFetchDigout2        = 0xfc;
        }
    }

    /**
     * Canandcolor bitsets.
     */
    public static class Bitsets {
        private Bitsets() {}

        /**
         * setting_report_flags - Whether the setting set/fetch was successful
         */
        public static final int kSettingReportFlags_SetSuccess = 0x1;

        /**
         * setting_report_flags - Whether the setting synch commit was successful
         */
        public static final int kSettingReportFlags_CommitSuccess = 0x2;

        /**
         * faults - The power cycle fault flag, which is set to true when the device first boots.
         * Clearing sticky faults and then checking this flag can be used to determine if the device rebooted.
         */
        public static final int kFaults_PowerCycle = 0x1;

        /**
         * faults - The CAN ID conflict flag, which is set to true if there is a CAN id conflict.
         * In practice, you should physically inspect the device to ensure it's not flashing blue.
         */
        public static final int kFaults_CanIdConflict = 0x2;

        /**
         * faults - The CAN general error flag, which will raise if the device encounters a CAN fault during operation.
         * If communication with the device still functions, this will not register as an active fault for long if at all.
         * This may raise due to wiring issues, such as an intermittently shorted CAN bus.
         */
        public static final int kFaults_CanGeneralError = 0x4;

        /**
         * faults - The temperature range flag, which will raise if the device is not between 0-70 degrees Celsius.
         * This may be of concern if the device is near very active motors.
         */
        public static final int kFaults_OutOfTemperatureRange = 0x8;

        /**
         * faults - The hardware fault flag corresponding to the distance sensor IC, which will raise if a hardware issue is detected.
         * Generally will raise if the device's controller cannot read the physical sensor itself.
         */
        public static final int kFaults_HardwareFaultDistance = 0x10;

        /**
         * faults - The hardware fault flag corresponding to the color sensor IC, which will raise if a hardware issue is detected.
         * Generally will raise if the device's controller cannot read the physical sensor itself.
         */
        public static final int kFaults_HardwareFaultColor = 0x20;

        /**
         * faults - The I2C bus recovery flag, which will raise when the device needs to completely restart the I2C bus.
         * This fault flag should not be active for very long; if it is stuck as an active fault, that may indicate a hardware issue.
         */
        public static final int kFaults_I2cBusRecovery = 0x40;

        /**
         * digout_cond - Slot 0
         */
        public static final int kDigoutCond_Slot0 = 0x1;

        /**
         * digout_cond - Slot 1
         */
        public static final int kDigoutCond_Slot1 = 0x2;

        /**
         * digout_cond - Slot 2
         */
        public static final int kDigoutCond_Slot2 = 0x4;

        /**
         * digout_cond - Slot 3
         */
        public static final int kDigoutCond_Slot3 = 0x8;

        /**
         * digout_cond - Slot 4
         */
        public static final int kDigoutCond_Slot4 = 0x10;

        /**
         * digout_cond - Slot 5
         */
        public static final int kDigoutCond_Slot5 = 0x20;

        /**
         * digout_cond - Slot 6
         */
        public static final int kDigoutCond_Slot6 = 0x40;

        /**
         * digout_cond - Slot 7
         */
        public static final int kDigoutCond_Slot7 = 0x80;

        /**
         * digout_cond - Slot 8
         */
        public static final int kDigoutCond_Slot8 = 0x100;

        /**
         * digout_cond - Slot 9
         */
        public static final int kDigoutCond_Slot9 = 0x200;

        /**
         * digout_cond - Slot 10
         */
        public static final int kDigoutCond_Slot10 = 0x400;

        /**
         * digout_cond - Slot 11
         */
        public static final int kDigoutCond_Slot11 = 0x800;

        /**
         * digout_cond - Slot 12
         */
        public static final int kDigoutCond_Slot12 = 0x1000;

        /**
         * digout_cond - Slot 13
         */
        public static final int kDigoutCond_Slot13 = 0x2000;

        /**
         * digout_cond - Slot 14
         */
        public static final int kDigoutCond_Slot14 = 0x4000;

        /**
         * digout_cond - Slot 15
         */
        public static final int kDigoutCond_Slot15 = 0x8000;

        /**
         * Constructs a setting_report_flags bitset.
         * 
         * @param setSuccess Whether the setting set/fetch was successful
         * @param commitSuccess Whether the setting synch commit was successful
         * @return bitset data as int
         */
        public static int constructSettingReportFlags(boolean setSuccess, boolean commitSuccess) {
                return (setSuccess ? 0x1 : 0) | 
                (commitSuccess ? 0x2 : 0);
        }

        /**
         * Constructs a faults bitset.
         * 
         * @param powerCycle The power cycle fault flag, which is set to true when the device first boots.
         * Clearing sticky faults and then checking this flag can be used to determine if the device rebooted.
         * @param canIdConflict The CAN ID conflict flag, which is set to true if there is a CAN id conflict.
         * In practice, you should physically inspect the device to ensure it's not flashing blue.
         * @param canGeneralError The CAN general error flag, which will raise if the device encounters a CAN fault during operation.
         * If communication with the device still functions, this will not register as an active fault for long if at all.
         * This may raise due to wiring issues, such as an intermittently shorted CAN bus.
         * @param outOfTemperatureRange The temperature range flag, which will raise if the device is not between 0-70 degrees Celsius.
         * This may be of concern if the device is near very active motors.
         * @param hardwareFaultDistance The hardware fault flag corresponding to the distance sensor IC, which will raise if a hardware issue is detected.
         * Generally will raise if the device's controller cannot read the physical sensor itself.
         * @param hardwareFaultColor The hardware fault flag corresponding to the color sensor IC, which will raise if a hardware issue is detected.
         * Generally will raise if the device's controller cannot read the physical sensor itself.
         * @param i2cBusRecovery The I2C bus recovery flag, which will raise when the device needs to completely restart the I2C bus.
         * This fault flag should not be active for very long; if it is stuck as an active fault, that may indicate a hardware issue.
         * @return bitset data as int
         */
        public static int constructFaults(boolean powerCycle, boolean canIdConflict, boolean canGeneralError, boolean outOfTemperatureRange, boolean hardwareFaultDistance, boolean hardwareFaultColor, boolean i2cBusRecovery) {
                return (powerCycle ? 0x1 : 0) | 
                (canIdConflict ? 0x2 : 0) | 
                (canGeneralError ? 0x4 : 0) | 
                (outOfTemperatureRange ? 0x8 : 0) | 
                (hardwareFaultDistance ? 0x10 : 0) | 
                (hardwareFaultColor ? 0x20 : 0) | 
                (i2cBusRecovery ? 0x40 : 0);
        }

        /**
         * Constructs a digout_cond bitset.
         * 
         * @param slot0 Slot 0
         * @param slot1 Slot 1
         * @param slot2 Slot 2
         * @param slot3 Slot 3
         * @param slot4 Slot 4
         * @param slot5 Slot 5
         * @param slot6 Slot 6
         * @param slot7 Slot 7
         * @param slot8 Slot 8
         * @param slot9 Slot 9
         * @param slot10 Slot 10
         * @param slot11 Slot 11
         * @param slot12 Slot 12
         * @param slot13 Slot 13
         * @param slot14 Slot 14
         * @param slot15 Slot 15
         * @return bitset data as int
         */
        public static int constructDigoutCond(boolean slot0, boolean slot1, boolean slot2, boolean slot3, boolean slot4, boolean slot5, boolean slot6, boolean slot7, boolean slot8, boolean slot9, boolean slot10, boolean slot11, boolean slot12, boolean slot13, boolean slot14, boolean slot15) {
                return (slot0 ? 0x1 : 0) | 
                (slot1 ? 0x2 : 0) | 
                (slot2 ? 0x4 : 0) | 
                (slot3 ? 0x8 : 0) | 
                (slot4 ? 0x10 : 0) | 
                (slot5 ? 0x20 : 0) | 
                (slot6 ? 0x40 : 0) | 
                (slot7 ? 0x80 : 0) | 
                (slot8 ? 0x100 : 0) | 
                (slot9 ? 0x200 : 0) | 
                (slot10 ? 0x400 : 0) | 
                (slot11 ? 0x800 : 0) | 
                (slot12 ? 0x1000 : 0) | 
                (slot13 ? 0x2000 : 0) | 
                (slot14 ? 0x4000 : 0) | 
                (slot15 ? 0x8000 : 0);
        }

        /**
         * Extracts set_success from setting_report_flags.
         * 
         * @param field data bitfield
         * @return true if set, false if not
         */
        public static boolean extractSettingReportFlags_SetSuccess(int field) {
            return (field & kSettingReportFlags_SetSuccess) > 0;
        }

        /**
         * Extracts commit_success from setting_report_flags.
         * 
         * @param field data bitfield
         * @return true if set, false if not
         */
        public static boolean extractSettingReportFlags_CommitSuccess(int field) {
            return (field & kSettingReportFlags_CommitSuccess) > 0;
        }

        /**
         * Extracts power_cycle from faults.
         * 
         * @param field data bitfield
         * @return true if set, false if not
         */
        public static boolean extractFaults_PowerCycle(int field) {
            return (field & kFaults_PowerCycle) > 0;
        }

        /**
         * Extracts can_id_conflict from faults.
         * 
         * @param field data bitfield
         * @return true if set, false if not
         */
        public static boolean extractFaults_CanIdConflict(int field) {
            return (field & kFaults_CanIdConflict) > 0;
        }

        /**
         * Extracts can_general_error from faults.
         * 
         * @param field data bitfield
         * @return true if set, false if not
         */
        public static boolean extractFaults_CanGeneralError(int field) {
            return (field & kFaults_CanGeneralError) > 0;
        }

        /**
         * Extracts out_of_temperature_range from faults.
         * 
         * @param field data bitfield
         * @return true if set, false if not
         */
        public static boolean extractFaults_OutOfTemperatureRange(int field) {
            return (field & kFaults_OutOfTemperatureRange) > 0;
        }

        /**
         * Extracts hardware_fault_distance from faults.
         * 
         * @param field data bitfield
         * @return true if set, false if not
         */
        public static boolean extractFaults_HardwareFaultDistance(int field) {
            return (field & kFaults_HardwareFaultDistance) > 0;
        }

        /**
         * Extracts hardware_fault_color from faults.
         * 
         * @param field data bitfield
         * @return true if set, false if not
         */
        public static boolean extractFaults_HardwareFaultColor(int field) {
            return (field & kFaults_HardwareFaultColor) > 0;
        }

        /**
         * Extracts i2c_bus_recovery from faults.
         * 
         * @param field data bitfield
         * @return true if set, false if not
         */
        public static boolean extractFaults_I2cBusRecovery(int field) {
            return (field & kFaults_I2cBusRecovery) > 0;
        }

        /**
         * Extracts slot0 from digout_cond.
         * 
         * @param field data bitfield
         * @return true if set, false if not
         */
        public static boolean extractDigoutCond_Slot0(int field) {
            return (field & kDigoutCond_Slot0) > 0;
        }

        /**
         * Extracts slot1 from digout_cond.
         * 
         * @param field data bitfield
         * @return true if set, false if not
         */
        public static boolean extractDigoutCond_Slot1(int field) {
            return (field & kDigoutCond_Slot1) > 0;
        }

        /**
         * Extracts slot2 from digout_cond.
         * 
         * @param field data bitfield
         * @return true if set, false if not
         */
        public static boolean extractDigoutCond_Slot2(int field) {
            return (field & kDigoutCond_Slot2) > 0;
        }

        /**
         * Extracts slot3 from digout_cond.
         * 
         * @param field data bitfield
         * @return true if set, false if not
         */
        public static boolean extractDigoutCond_Slot3(int field) {
            return (field & kDigoutCond_Slot3) > 0;
        }

        /**
         * Extracts slot4 from digout_cond.
         * 
         * @param field data bitfield
         * @return true if set, false if not
         */
        public static boolean extractDigoutCond_Slot4(int field) {
            return (field & kDigoutCond_Slot4) > 0;
        }

        /**
         * Extracts slot5 from digout_cond.
         * 
         * @param field data bitfield
         * @return true if set, false if not
         */
        public static boolean extractDigoutCond_Slot5(int field) {
            return (field & kDigoutCond_Slot5) > 0;
        }

        /**
         * Extracts slot6 from digout_cond.
         * 
         * @param field data bitfield
         * @return true if set, false if not
         */
        public static boolean extractDigoutCond_Slot6(int field) {
            return (field & kDigoutCond_Slot6) > 0;
        }

        /**
         * Extracts slot7 from digout_cond.
         * 
         * @param field data bitfield
         * @return true if set, false if not
         */
        public static boolean extractDigoutCond_Slot7(int field) {
            return (field & kDigoutCond_Slot7) > 0;
        }

        /**
         * Extracts slot8 from digout_cond.
         * 
         * @param field data bitfield
         * @return true if set, false if not
         */
        public static boolean extractDigoutCond_Slot8(int field) {
            return (field & kDigoutCond_Slot8) > 0;
        }

        /**
         * Extracts slot9 from digout_cond.
         * 
         * @param field data bitfield
         * @return true if set, false if not
         */
        public static boolean extractDigoutCond_Slot9(int field) {
            return (field & kDigoutCond_Slot9) > 0;
        }

        /**
         * Extracts slot10 from digout_cond.
         * 
         * @param field data bitfield
         * @return true if set, false if not
         */
        public static boolean extractDigoutCond_Slot10(int field) {
            return (field & kDigoutCond_Slot10) > 0;
        }

        /**
         * Extracts slot11 from digout_cond.
         * 
         * @param field data bitfield
         * @return true if set, false if not
         */
        public static boolean extractDigoutCond_Slot11(int field) {
            return (field & kDigoutCond_Slot11) > 0;
        }

        /**
         * Extracts slot12 from digout_cond.
         * 
         * @param field data bitfield
         * @return true if set, false if not
         */
        public static boolean extractDigoutCond_Slot12(int field) {
            return (field & kDigoutCond_Slot12) > 0;
        }

        /**
         * Extracts slot13 from digout_cond.
         * 
         * @param field data bitfield
         * @return true if set, false if not
         */
        public static boolean extractDigoutCond_Slot13(int field) {
            return (field & kDigoutCond_Slot13) > 0;
        }

        /**
         * Extracts slot14 from digout_cond.
         * 
         * @param field data bitfield
         * @return true if set, false if not
         */
        public static boolean extractDigoutCond_Slot14(int field) {
            return (field & kDigoutCond_Slot14) > 0;
        }

        /**
         * Extracts slot15 from digout_cond.
         * 
         * @param field data bitfield
         * @return true if set, false if not
         */
        public static boolean extractDigoutCond_Slot15(int field) {
            return (field & kDigoutCond_Slot15) > 0;
        }
    }
}