// Copyright (c) Redux Robotics and other contributors.
// This is open source and can be modified and shared under the 3-clause BSD license.

#pragma once
#include <stdint.h>

namespace redux::canand {
/**
 * Class representing CAN buses that may exist on a robot.
 * 
 * <b>Currently, only the Rio's onboard CAN bus is supported.</b>
 * 
*/
class CANBus {
  public:

    /**
     * Constructs a new CAN bus from a descriptor value. 0 is the Rio's onboard bus.
     * @param descriptorId the CAN bus descriptor value.
    */
    inline CANBus(uint8_t descriptorId) { fd = descriptorId; }
    virtual ~CANBus() = default;

    /**
     * Returns the descriptor ID associated with the CAN bus object. 
     * Generally not needed to be used directly.
     * @return the descriptor ID.
     */
    inline uint8_t GetDescriptor() { return fd; }

    /**
     * Returns whether two CANBus objects refer to the same bus.
     * @param other other CANBus object to compare against
     * @return whether or not they refer to the same bus
     */
    inline bool Equals(CANBus other) { return GetDescriptor() == other.GetDescriptor(); }
  private:
    uint8_t fd;
};
}